/*H+ HotRegion.h
*
*    SCCS ID: @(#)HotRegion.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE: 
*
*    Defines the HotRegion class.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    MGS       02/10/91   Original creation
*    JDZ       04/29/91   Preparation for code review
*
*H-*/

/*C+ HotRegion
*
*    DESCRIPTION:
*
*    The HotRegion class allows specification of geometrical regions
*    that are sensitive to events.  The default geometrical region
*    is a rectangle.  The user can define geometrical regions by specifying
*    a detection function of their own.
*
*    INHERITED CLASSES:
*
*    Generic - Inheriting the Generic class will allow the HotRegion class
*    to be treated as a Generic object. Therefore, a GenericHash or
*    GenericList class can contain a HotRegion.
*
*    ORDINARY DATA MEMBERS:
*
*    int                        defaultX          - Defines the X coordinate 
*    of the default rectangle region that defines the HotRegion.
*
*    int                        defaultY          - Defines the Y coordinate
*    of the default rectangle region that defines the HotRegion.
*
*    int                        defaultWidth      - Defines the width of the 
*    default rectangle region that defines the HotRegion.
*
*    int                        defaultHeight     - Defines the height of the
*    default rectangle region that defines the HotRegion.
*
*    int                        hotRegionID       - Holds the user specified
*    ID of the hot region.
*
*    Boolean                    transparent       - FALSE if this region
*    blocks events going to the region below it.
*
*    UIObject                  *hotRegionObject   - Holds the pointer of the 
*    object this region is attached to.
*
*    UICHandlerFunction         regionHandler     - User defined function to 
*    execute when an event occurs in this hot region.
*
*    HotRegionDetectionFunction detectionFunction - User defined function to 
*    detect if an event is in the hot region.  The user's function must
*    receive the X-Y coordinates of the event and return TRUE or FALSE.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*    void      setDefaultRectangle (int x, int y, int width, int height) - Sets
*    the default rectangle that defines the HotRegion.  Default: Depends
*    on constructor.
*
*    void      setTransparent (Boolean transparent_flag)         - Sets the 
*    transparent rectangle that defines the HotRegion (i.e., it blocks
*    events going to the region below it).  Default: FALSE.
*
*    void      setDetectionFunction (HotRegionDetectionFunction) - Sets the
*    detectionFunction on this HotRegion object.  Default: Rectangular
*    region detection.
*
*    void      setRegionHandler (UICHandlerFunction)             - Sets the
*    function that when an event occurs in this hot region.  Default: Not set.
*
*    UIObject *getHotRegionObject ()                             - Returns the
*    pointer of the object this hot region is attached to.
*
*    Boolean   isHotRegion (int, int, UIObject *)                - TRUE if
*    this location belongs to a HotRegion object.
*
*    Boolean   isTransparent ()                                  - TRUE if
*    this HotRegion object is transparent (i.e., it blocks events going to the 
*    region below it).
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    Boolean defaultDetectionFunction (int x, int y) -  Checks if the X-Y 
*    coordinates fall within the default rectangular region.
*
*C-*/
#ifndef _HOTREGION_H
#define _HOTREGION_H

// Includes
#include "UIObject.h"

// Type Definitions
typedef Boolean (* HotRegionDetectionFunction) (int, int);

// Class Definition
class HotRegion : public UIObject {
public:
  HotRegion ()
    {
      defaultX = UIC_NOT_SET;
      defaultY = UIC_NOT_SET;
      defaultWidth = UIC_NOT_SET;
      defaultHeight = UIC_NOT_SET;
      hotRegionID = UIC_NOT_SET;
      transparent = FALSE;
    }
  HotRegion (int id)
    {
      defaultX = UIC_NOT_SET;
      defaultY = UIC_NOT_SET;
      defaultWidth = UIC_NOT_SET;
      defaultHeight = UIC_NOT_SET;
      hotRegionID = id;
      transparent = FALSE;
    }
  HotRegion (int id,
	   int x,
	   int y,
	   int width,
	   int height,
	   Boolean transparent_flag)
    {
      defaultX = x;
      defaultY = y;
      defaultWidth = width;
      defaultHeight = height;
      hotRegionID = id;
      transparent = transparent_flag;
      hotRegionObject = (UIObject *) NULL;
      regionHandler = UIC_NO_FUNCTION;
    }
  ~HotRegion () {}
  int       getID () { return hotRegionID; }
  void      setID (int id) { hotRegionID = id; }
  void      getDefaultRectangle (int &x, int &y, int &width, int &height)
    {
      x = defaultX;
      y = defaultY;
      width = defaultWidth;
      height = defaultHeight;
    }
  void      setDefaultRectangle (int x, int y, int width, int height)
    {
      defaultX = x;
      defaultY = y;
      defaultWidth = width;
      defaultHeight = height;
    }
  void      setTransparent (Boolean transparent_flag)
    { transparent = transparent_flag; }

  void      setDetectionFunction (HotRegionDetectionFunction); 
  void      setRegionHandler (UICHandlerFunction);

  UIObject *getHotRegionObject () { return hotRegionObject; }

  Boolean   isHotRegion (int, int, UIObject *);
  Boolean   isTransparent () { return transparent; }

protected:
  int                 defaultX;
  int                 defaultY;
  int                 defaultWidth;
  int                 defaultHeight;
  int                 hotRegionID;
  Boolean             transparent;
  UIObject           *hotRegionObject;
  UICHandlerFunction  regionHandler;

  HotRegionDetectionFunction detectionFunction;

  Boolean                    defaultDetectionFunction (int x, int y)
    {
      Boolean result = FALSE;

      if ((defaultX == UIC_NOT_SET) &&
	  (defaultY == UIC_NOT_SET) &&
	  (defaultWidth == UIC_NOT_SET) &&
	  (defaultHeight == UIC_NOT_SET) &&
	  (transparent == FALSE))
        result = FALSE;
      else if (((x >= defaultX) && (x <= defaultX + defaultWidth)) && 
               ((y >= defaultY) && (y <= defaultY + defaultHeight)))
	result = TRUE;

      return result;
    }
};

#endif /* _HOTREGION_H */
