/*H+ ScrollChoice.h
*
*    SCCS ID: @(#)ScrollChoice.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the ScrollChoice class.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    MGS       05/10/91   Prepare for code review.
*
*H-*/

/*C+ ScrollChoice
*
*    DESCRIPTION:
*
*    A ScrollChoice object displays a list of various text and/or image fields
*    in a scrolling region.  A user may select one or more entries from the
*    the list depending on whether or not it was specified as a multiple
*    choice list.  This class has most of the ListChoice classes capabilities.
*    The main difference is the way user selections are shown.  The ListChoice
*    class shows it as a 3D depression, whereas this class shows the selected
*    items with a bold font.  The only other difference is that the XView
*    edit menu is not supported by the ScrollChoice class.
*
*    The reason for this class was to create a work around for bugs found
*    in XView V2 related to the PANEL_LIST package.  The PANEL_LIST bugs
*    have been fixed in XView V3 and this class will no longer be needed
*    once V3 is being used.
*
*    INHERITED CLASSES:
*
*    UIChoice - Abstract choice class.
*
*    LOCAL CLASSES:
*
*    ListClientData - The ListClientData class is used to store a unique
*                     row ID and the UIC image corresponding to each row
*                     in the list.  The structure is attached to the row
*                     via the XView PANEL_LIST_CLIENT_DATA attribute.
*
*    ORDINARY DATA MEMBERS:
*
*    int	      rowHeight;         - The pixel height of a row.
*    int	      displayedRows;     - The number rows to be displayed.
*    int	      internalSelection; - Because the ScrollChoice class
*                                          handles its own selection tracking,
*                                          this is used to keep track of the
*                                          selection actually selected at each
*                                          list operation for use by the
*                                          internal list notify algorithm.
*    Boolean	      selectionRequired; - Indicates whether or not a list
*                                          selection is required.
*    Boolean	      readOnly;          - For a ListChoice class, this
*                                          specifies whether or not the user
*                                          can edit the list via the default
*                                          XView list menu.  Since the
*                                          Scrollchoice doesn't use the XView
*                                          edit menu, this data memeber is
*                                          meaningless.  Is kept for ListChoice
*                                          member function compatibility.
*    SelectionStatus  selectStatus;      - Saves the selection status for the
*                                          last operation on the list.
*    UIMenu	     *listMenu;          - Pointer to a user specified menu.
*    int              rowID;             - Used for generating unique row IDs.
*    CharacterFont   *labelFont;         - Pointer to the user specified font
*                                          for the panel list label.
*    CharacterFont   *listFont;          - Pointer to the user specified font
*                                          for panel list row text.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*    void setRowHeight (int height) - Sets the pixel height of a row.
*                                     Default: XView default.
*
*    void setDisplayedRows (int num_rows)
*         - Sets the number of rows to be displayed.  Default: 5.
*
*    void setReadOnly (Boolean readWrite)
*         - See the readOnly data member.  Default: FALSE.
*
*    void setSelectionRequired (Boolean selection)
*         - Specifies whether or not a selection is required.  If set on,
*           at least one selection will always be set.  Default: FALSE.
*
*    void setLabelFont (CharacterFont &font)
*         - Specifies the font to be used for the label.
*           Default: Not set.
*
*    void setListFont (CharacterFont &font)
*         - Specifies the font to be used for row text.
*           Default: Not set.
*
*    void setMenu (UIMenu &)
*         - Allows specification of a menu to be shown when the mouse menu
*           button is pressed.
*           Default: Not set.
*
*    SelectionStatus getOperation ()
*         - Gets the the last operation performed on the list.
*
*    void deleteAllChoices ()
*         - Deletes all of the choices in the list.
*
*    virtual void deleteChoice (int)
*    virtual void deleteChoice (char *)
*    virtual void deleteChoice (Image &)
*            - Deletes the specified choice from the list.  The choice
*              may be specified by row position, text value or image value.
*
*    virtual int addChoice (char *)
*    virtual int addChoice (Image &)
*    virtual int addChoice (char *, Image &)
*            - Adds the specified text and/or image value as the last choice
*              in the list.
*
*    virtual int addChoice (int, char *)
*    virtual int addChoice (int, Image &)
*    virtual int addChoice (int, char *, Image &)
*            - Adds the specified text and/or image value at the specified
*              row position in the list.
*
*    virtual Image *getChoice (Image* &)
*    virtual char  *getChoice (char* &)
*    virtual int    getChoice (int &)
*            - Returns the text value, image value or row position of the
*              last choice operated on.
*
*    virtual char  *getChoice (int, char* &)
*    virtual Image *getChoice (int, Image* &)
*    virtual int    getChoice (int, int &)
*            - Returns the text value, image value or row position of the
*              specified selected choice.  The choice is specified by
*              its position in the list of selected items in this object.
*
*    virtual int getNumberSelected ()
*            - Returns the number of currently selected items.
*
*    virtual char  *getValue (char* &)
*    virtual Image *getValue (Image* &)
*            - Returns the text value, image value of the last choice
*              operated on.
*    
*    virtual char  *getValue (int, char* &)
*    virtual Image *getValue (int, Image* &)
*            - Returns the text value, image value of the specified
*              choice.  The choice is specified by its row position in
*              the list.
*
*    void replaceChoice (int, char *)
*    void replaceChoice (int, Image &)
*    void replaceChoice (int, char *, Image &)
*         - Replaces the text and/or image value of the specified choice.
*           The choice is specified by its row position in the list.
*
*    int getNumberOfChoices ()
*        - Returns the number of choices in the list.
*
*    virtual void setSelectedChoice (int, Boolean flag = TRUE)
*    virtual void setSelectedChoice (char *, Boolean flag = TRUE)
*    virtual void setSelectedChoice (Image &, Boolean flag = TRUE)
*        - Sets the specified choice either on or off.  The choice can
*          be specified by text value, image value, or row position.
*
*    virtual Boolean isSelectedChoice (int row_number)
*        - Determines if the specified choice is selected.
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    int notifyProc ()
*         - The ScrollChoice notify proc wrapper.
*    void scrollChoiceEventProc (Panel_item, Event *)
*         - The heart of the ScrollChoice class.  This handles all the
*           XView events inrelation to ScrollChoice objects.
*
*    virtual void showFeedback (int, Boolean)
*         - Shows selection feedback for the specified choice.
*
*    PRIVATE MEMBER FUNCTIONS:
*
*    void setScrollChoiceDefaults ()
*         - Initializes the data memebers.
*
*C-*/
#ifndef _SCROLL_CHOICE_H
#define _SCROLL_CHOICE_H

// Includes
#include "CharacterFont.h"
#include "UIChoice.h"
#include "UIMenu.h"

// Local Class Definitions
class ListClientData : public Generic {
public:
  ListClientData (int row_id, Image *image_pointer) { rowID = row_id;
					              image = image_pointer; }
  int    rowID;
  Image *image;
};

// Class Definition
class ScrollChoice : public UIChoice {
public:
 
			 ScrollChoice ();
    			 ScrollChoice (char *);
			 ScrollChoice (Image &);
			~ScrollChoice ();

    void		 setRowHeight (int height)
			 { rowHeight = height;
			   if (owObject) xv_set (owObject,
						 PANEL_LIST_ROW_HEIGHT,
						 rowHeight,
						 NULL); }

    void		 setDisplayedRows (int num_rows)
			 { displayedRows = num_rows;
			   if (owObject) xv_set (owObject, 
			   		         PANEL_LIST_DISPLAY_ROWS,
						 displayedRows,
						 NULL); }

    void		 setReadOnly (Boolean readWrite)
		 	 { readOnly = readWrite;
			   if (owObject)
			     xv_set (owObject,
				     PANEL_READ_ONLY, readWrite,
				     NULL); }

    void                 setLabelFont (CharacterFont &font)
			 { labelFont = &font;
			   if (owObject) {
			     ((UIObject *) labelFont)->createObject (this);
			     xv_set (owObject,
				     PANEL_LABEL_FONT,
				     labelFont->getXViewObject (),
				     NULL);
			   }
			 }

    void                 setListFont (CharacterFont &font)
			 { if (!owObject) listFont = &font; }

    void		 setMenu (UIMenu &);

    void		 setSelectionRequired (Boolean flag)
			 { selectionRequired = flag; }

    virtual Boolean      isSelectedChoice (int);

    SelectionStatus	 getOperation ()
			 { return (selectStatus); }

    void		 deleteAllChoices ();

    virtual void	 deleteChoice (int);
    virtual void	 deleteChoice (char *);
    virtual void	 deleteChoice (Image &);

    virtual int		 addChoice (char *);
    virtual int	 	 addChoice (Image &);
    virtual int		 addChoice (char *, Image &);

    virtual int 	 addChoice (int, char *);
    virtual int	 	 addChoice (int, Image &);
    virtual int	 	 addChoice (int, char *, Image &);

    virtual Image	*getChoice (Image *&);
    virtual char	*getChoice (char *&);
    virtual int	 	 getChoice (int &);

    virtual int		 getNumberSelected ();

    virtual char	*getChoice (int, char *&);
    virtual int 	 getChoice (int, int &);
    virtual Image	*getChoice (int, Image *&);

    void		 replaceChoice (int, char *);
    void	 	 replaceChoice (int, Image &);
    void	 	 replaceChoice (int, char *, Image &);

    virtual char        *getValue (char* &retval);
    virtual Image       *getValue (Image* &retval);

    virtual char        *getValue (int, char* &retval);
    virtual Image       *getValue (int, Image* &retval);

    int			 getNumberOfChoices ()
			 { int num_choices;
			   if (owObject)
			     num_choices = (int) xv_get (owObject,
						         PANEL_LIST_NROWS,
						         NULL);
			   else
			     num_choices = choices.getSize ();
			   return num_choices; }

    virtual void	 setSelectedChoice (int, Boolean);
    virtual void	 setSelectedChoice (char *, Boolean);
    virtual void	 setSelectedChoice (Image &, Boolean);

protected:
    int			 rowHeight;
    int			 displayedRows;
    int			 internalSelection;
    int			 rowID;
    Boolean		 selectionRequired;
    Boolean		 readOnly;
    SelectionStatus	 selectStatus;
    UIMenu		*listMenu;
    CharacterFont       *labelFont;
    CharacterFont       *listFont;

    virtual void	 createObject (UIObject *);
    virtual void         showFeedback (int, Boolean);

    void	         notifyProc ();

    static void		 scrollChoiceEventProc (Panel_item, Event *);

private:
    void		 setScrollChoiceDefaults ();
};

#endif /* _SCROLL_CHOICE_H */
