/*H+ UIComponent.h 
*
*    SCCS ID: @(#)UIComponent.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE: 
*
*    Defines the UIComponent class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    JDZ       05/06/91   Preparation for code review
*
*H-*/

/*C+ UIComponent
*
*    DESCRIPTION:
*
*    Defines the generic characteristics of UIComponents (Button, Slider,
*    ToggleChoice, etc...) in a GUI application. These include such things
*    as the layout orientation (horizontal or vertical) ways of object label
*    represenation (image or string).
*
*    INHERITED CLASSES:
*
*    UIObject - The base class of all UI classes in the UIC.
*
*    ORDINARY DATA MEMBERS:
*
*    Image	 	 *imageLabel     - Pointer to the label that
*    represents object. Could be either an image or a character.
*
*    int		  valueX         - The X coordinate of the location of 
*    the value associated with this UIComponent within the ComponentDisplay.
*
*    int		  valueY         - The Y coordinate of the location of
*    the value associated with this UIComponent within the ComponentDisplay.
*
*    Boolean              active         - TRUE if this component button
*    is active.
*
*    Position		  labelPosition  - Holds the location of the label 
*    associated with this UIComponent.  Position is a data type that has two 
*    values: LEFT and ABOVE.
*
*    int                  notifyResult   - User specified result of the last
*    notify event on this object.
*
*    CharacterFont       *labelFont      - The font to be used to render the
*    component label.
*
*    CharacterFont       *valueFont      - The font to be used to render the
*    component value.
*
*    GenericList	  notifyTargets  - The list of targets to be notified
*    when a notify event occurs on this object.
*
*    UICHandlerFunction   notifyHandler  - Internal wrapper around an XView
*    notify proc for objects of classes derived from UIComponent.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*           void    setLabel (char *) - Sets the text label of this object.
*           Default: Not set.
*
*           void    setLabel (Image&) - Sets the image label of this object.
*           Default: Not set.
*
*           void    setLabelPosition (Position) - Sets the location for the
*           label associated with this object.  Default: Not set.
*
*           void    setValueX (int x) - Sets the X coordinate of the location
*           of the value associated with this UIComponent within the
*	    ComponentDisplay.  Default: Not set.
*
*           void    setValueY (int y) - Sets the Y coordinate of the location
*           of the value associated with this UIComponent within the
*	    ComponentDisplay.  Default: Not set.
*
*           void    setActive (Boolean) - Designates this component as
*           active.  Default: Active.
*
*           void    setNotifyHandler (UICHandlerFunction) - Sets the function
*	    that is executed whenever an event occurred in this component.
*           Default: Not set.
*
*           void    setNotifyResult (int result) - Allws the user to specify
*           the result of a notify event.  Default: XV_OK.
*
*           void    setNotifyResult (DisplaySetting result) - Allows the user
*           to specify the result of a notify event.  Default: The return
*           value of the XView "panel_text_notify" function.
*
*           void    setLabelFont (CharacterFont &font) - Sets the character
*	    font for this UIComponent's label.  Default: Not set.
*
*           void    setValueFont (CharacterFont &font) - Sets the character
*	    font for this UIComponent's value.  Default: Not set.
*
*           int     addNotifyTarget (UIObject&, UICTargetFunction) - Adds
*	    the specified object as a target to be notified when the
*	    UIComponent is notified.  The target function is called with
*	    the pointer to the UIComponent and the target object.
*
*   virtual void    setForegroundColor (Color)
*   virtual void    setForegroundColor (char *) -
*	    Sets the foreground color of the component.  The color is
*	    specified by either name or a UIC color structure containing
*	    the RGB components of the color. 
* 
*   virtual Boolean isScroller () - Determines whether or not this component
*	    is a Scroller object.	
*
*   virtual Boolean isDrawableOnCanvas () - Virtual function that determines
*   whether or not a component can be drawn on an XView canvas.  This is
*   provided for use by user defined components.  If user defined component
*   does not have to be on an XView panel and can be displayed on an XView
*   canvas, this function should be defined to return TRUE.  The UIComponent
*   class takes care of the default return of FALSE.
*
*   virtual void    repaintComponent (ComponentDisplay *) - Virtual function 
*   which is implemented in a user-defined component.  This function is
*   called by ComponentDisplay whenever the component needs to be repainted.
*
*   virtual void    componentHotRegionEventProc (ComponentDisplay *) - Virtual
*   function which is implemented in a user-defined component.  This function
*   is called by ComponentDisplay whenever an event occurs inside of a user-
*   defined component's hot region.
*
*   virtual void    componentBackgroundEventProc (ComponentDisplay *) - Virtual
*   function which is implemented in a user-defined component.  This function
*   is called by ComponentDisplay whenever an event occurs inside the
*   ComponentDisplay, but outside of the component's hot region.
*
*   PROTECTED MEMBER FUNCTIONS:
*
*   static  void   eventProc (Panel_item, Event *) - Internal wrapper
*           around an XView event proc.
*
*           void   notifyNotifyTargets () - Notifies all of the targets
*           associated with this component.
*
*   virtual void   createWithFonts (UIObject *, Xv_pkg *, void *) - Creates
*   the XView object for this object.  Takes into account user specified
*   fonts in the xv_create call. 
*
*   virtual void   createWithFonts (UIObject *,
*				    Xv_pkg *,
*				    Panel_attr, Panel_attribute_value,
*				    Panel_attr, Panel_attribute_value,
*                                   void *) - Creates the XView object
*   for this object.  Takes into account user specified fonts in the
*   xv_create call.  This version of createWithFonts expects the specification
*   of two XView attributes.
*
*           void   setUIComponentAttributes (UIObject *) - Sets XView
*   attributes that are common to the classes derived from UIComponent.
*
*   PRIVATE MEMBER FUNCTIONS:
*
*           void   setUIComponentDefaults () - Initializes data members.
*
*C-*/
#ifndef _UICOMPONENT_H
#define _UICOMPONENT_H

// Includes
#include "UIObject.h"
#include "CharacterFont.h"
#include "Image.h"

#include <xview/rect.h>
#include <xview/win_input.h>
#include <pixrect/pixrect.h>
#include <pixrect/pixfont.h>
#include <xview/panel.h>
#include <xview/scrollbar.h>

// Class Declarations
class ComponentDisplay;

// Class Definition
class UIComponent : public UIObject {
public:
    		  	 UIComponent ();
    		  	 UIComponent (char *);
		  	 UIComponent (Image &);
			~UIComponent ();

    void	  	 setLabel (char *);
    void	  	 setLabel (Image&);
    void                 setLabelPosition (Position);
    
    void		 setValueX (int x)
			 { valueX = x;
			   if (owObject) xv_set (owObject,
						 PANEL_VALUE_X, valueX,
						 NULL); }

    void		 setValueY (int y)
			 { valueY = y;
			   if (owObject) xv_set (owObject,
						 PANEL_VALUE_Y, valueY,
						 NULL); }

    void		 setActive (Boolean flag)
			 { active = flag;
			   if (owObject) {
			     if (isScroller ())
			       xv_set (owObject,
				       SCROLLBAR_INACTIVE,
				       (flag ? FALSE : TRUE),
				       NULL);
			     else
			       xv_set (owObject,
				       PANEL_INACTIVE, (flag ? FALSE : TRUE),
				       NULL);
			   }
			 }

    void                 setNotifyHandler (UICHandlerFunction);

    void                 setNotifyResult (int result)
			 { notifyResult = result; }

    void                 setNotifyResult (DisplaySetting result)
			 { notifyResult = (int) result; }

    void                 setLabelFont (CharacterFont &font)
			 { labelFont = &font;
			   if (owObject) {
			     ((UIObject *) labelFont)->createObject (this);
			     xv_set (owObject,
				     PANEL_LABEL_FONT,
				     labelFont->getXViewObject (),
				     NULL);
			   }
		         }

    void                 setValueFont (CharacterFont &font)
			 { valueFont = &font;
			   if (owObject) {
			     ((UIObject *) labelFont)->createObject (this);
			     xv_set (owObject,
				     PANEL_VALUE_FONT,
				     valueFont->getXViewObject (),
				     NULL);
		           }
                         }

    int		 	 addNotifyTarget (UIObject&, UICTargetFunction);

    virtual void         setForegroundColor (Color);
    virtual void         setForegroundColor (char *);
	 
    virtual Boolean	 isScroller ();

    virtual Boolean	 isDrawableOnCanvas ();

    virtual void	 repaintComponent (ComponentDisplay *);
    virtual void	 componentHotRegionEventProc (ComponentDisplay *);
    virtual void         componentBackgroundEventProc (ComponentDisplay *);

protected:
    Image	 	*imageLabel;
    int			 valueX;
    int			 valueY;
    Boolean		 active;
    Position		 labelPosition;
    int                  notifyResult;
    CharacterFont       *labelFont;
    CharacterFont       *valueFont;
    GenericList		 notifyTargets;
    UICHandlerFunction   notifyHandler;

    void		 notifyNotifyTargets ();

    virtual void	 createWithFonts (UIObject *, Xv_pkg *, void *);
    virtual void	 createWithFonts (UIObject *,
					  Xv_pkg *,
					  Panel_attr, Panel_attribute_value,
					  Panel_attr, Panel_attribute_value,
					  void *);

    void		 setUIComponentAttributes (UIObject *);

    static void	 	 eventProc (Panel_item, Event *);

private:
    void		 setUIComponentDefaults ();
};

#endif /* _UICOMPONENT_H */
