/*H+ UIObject.h
*
*    SCCS ID: @(#)UIObject.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE: 
*
*    Defines the UIObject class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    JDZ       04/26/91   Preparation for code review
*
*H-*/

/*C+ UIObject
*
*    DESCRIPTION:
*  
*    The nature of this object is to define the generic characteristics 
*    of all UIC objects.  It includes such things as setting the properties 
*    of an object, such as its height, width, color and the function that 
*    will execute when an event happens to this object, such as a user 
*    clicking a mouse on it.  It also carries virtual functions which the 
*    Derived Class implements. 
*
*    INHERITED CLASSES:
*
*    Generic - Inheriting the Generic class will allow the UIObject class
*    to be treated as a Generic object. Therefore, a GenericHash or 
*    GenericList class can contain a UIObject.
*
*    LOCAL CLASSES:
*
*    InputEvent - contains all of the information that pertains to a user
*    interface event, such as a user clicking the mouse pointer on a button.
*    Every UIC class has the ability to obtain the last event that happened
*    to an object, while in a handler, by using the getEvent member function.  
*    When your application calls the getEvent function, that function returns 
*    a pointer to an InputEvent object. You NEVER create an instance of this 
*    class; this object is always returned by the getEvent.  Observe, that 
*    this class has no corresponding .h file.
*
*             ORDINARY DATA MEMBERS:
*
*             int	  eventID     - the XView event ID.
*
*             int	  eventAction - the XView action.
*
*             Boolean     upFlag      - is TRUE if the mouse button 
*	      is released (not pressed down) when the event occurs.
*
*	      Boolean     downFlag    - is TRUE if the mouse button
*	      is pressed when the event occurs.
*
*	      Event      *xViewEvent  -  pointer to the XView
*	      Event structure.
*
*             PUBLIC MEMBER FUNCTIONS: 
*
*             int     getID()          - Returns the XView event ID.
*
*	      int     getAction()      - Returns the XView action.
*
*	      Boolean isButtonUp()     - Returns TRUE if the mouse button 
*	      is released (not pressed down) when the event occurs.
*
*	      Boolean isButtonDown()   - Returns TRUE if the mouse button
*	      is pressed when the event occurs.
*
*	      Event   *getXViewEvent() - Returns the pointer to the XView
*	      Event structure.
*
*             int      getX()    - Returns the X coordinate of the position 
*             of the locator (mouse) relative to the window in which the event 
*             occurred.
*
*             int      getY()    - Returns the Y coordinate of the position 
*             of the locator (mouse) relative to the window in which the event 
*             occurred.
*
*	      timeval *getTime() - Returns the absolute time of the event.
*
*   UITarget - contains information used to assocate/bind objects. This 
*   binding occurs between the object in which the event is noted and a list 
*   of objects that require "secondary" notifications.  These notifications,
*   in turn, may trigger some functions to be executed upon object-members of 
*   the list. This class is used internally by the UIObject only.  You NEVER 
*   create an instance of this class and there is no corresponding .h file.
*
*             ORDINARY DATA MEMBERS:
*
*             UIObject          *targetObject   - pointer to the
*             associated/binded object or list of objects.
*
*             UITargetFunction   targetFunction - function that is to be
*             executed upon the above object or list of objects when the
*	      triggering event is noted in the original object.
*             
*    STATIC DATA MEMBERS:
*
*    Display    *rootDisplay     - The X Display pointer of the root window.
*    int	 rootWindowWidth - The width of the frame buffer.
*    int	 rootWindowHeight - The Height of the frame buffer.
*
*    ColorMap   *colorMap        - The main color map used by all of the UIC 
*    objects.
*
*    Boolean     defaultColorMap - TRUE if the default color map is being used.
*
*    UIObject   *baseWindow      - Pointer to the main (first) Base Window.
*
*    int         numUIObjects    - The number of objects (derived from 
*    UIObject) instantiated.
*
*    int         uiObjectID      - Used for generating a unique ID for every 
*    object (derived from UIObject) instantiated.
*
*    ORDINARY DATA MEMBERS:
*
*    int                 xLoc               - X coordinate of this object.
*
*    int                 yLoc               - Y coordinate of this object.
*
*    int                 width              - Width of this object. 
*
*    int                 height             - Height of this object.
*
*    char               *label              - Pointer to the label that is to 
*    appear on the object. 
*
*    char               *help               - Specifies the help string used 
*    by the help package to display on-line help. Default: No help available.
*
*    UICGeneric          owObject           - XView object pointer.
*
*    int		 foregroundColor    - Numerical value of the Foreground
*    Color derived from the Color Map. 
*
*    int		 backgroundColor    - Numerical value of the Background
*    Color derived from the Color Map.
*
*    UICHandlerFunction	 eventHandler       - Function that is to be executed 
*    when an event is noted in this object. 
*
*    GenericList	 events             - List of events to consume.  Used
*    for specifing that the window will accept an event of the type specified.
*
*    InputEvent		 lastEvent          - The last event that occurred on 
*    this object. 
*
*    Boolean		 eventResultSet     - TRUE if the user had specified an 
*    event result, while in the handler.
*
*    EventResult	 eventResult        - Contains the event result the 
*    user specified.
*
*    GenericList	 hotRegions         - Lists all hot regions (active or 
*    not) associated with this object.
*
*    GenericList	 activeHotRegions   - Lists all active hot regions 
*    associated with this object.
*
*    Boolean		 hotRegionEventFlag - TRUE if the last event took 
*    place in a hot region.
*
*    GenericList	 eventTargets       - List of objects to be notified 
*    when an event occurs in this object.
*
*    CharacterFont      *font               - The font to be used with this 
*    object.
*
*    GC                  fontGC             - A holder for the X font graphics 
*    context for this object.
*
*    GenericHash        *objectData         - A table containing object data
*    associated with this object.
*
*    Boolean		 showFlag           - Specifies whether or not this 
*    object is to be shown (XV_SHOW).
*
*    int		 uiObjectID         - The object's unique ID.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*            void initUI (int &argc, char **argv,
*			  UICAttribute      attribute = UIC_NO_ATTRIBUTE,
*			  UICAttributeValue value = NULL, ...) -
*    In addition to initializing the graphics server, this function also is
*    used to specify attributes for the current session with the UIC.  The
*    only attribute currently implemented is UIC_OPEN_PS_CONNECTION.
*    UIC_OPEN_PS_CONNECTION can be set to TRUE or FALSE.  If TRUE, then the UIC
*    will open a PostScript connection to the NeWS server.  If set to FALSE,
*    then the UIC will not open the PostScript connection and many of the
*    PostScript member functions provided in the UIC ComponentDisplay class
*    will not work.  The following is an example of specifying the attribute as
*    TRUE:
*
*       	initUI (argc, argv, UIC_OPEN_PS_CONNECTION, TRUE, NULL);
*
*    If the attribute is not to be set, the following will default to the 
*    attribute being set FALSE:
*
*       	initUI (argc, argv);
*
*    const char *getVersion () - Returns the release compatibility
*				 version of the UIC.  Currently, all versions
*				 of the UIC that return "UICV1" are compatible.
*
*    int getRootWindowWidth () - Returns the width of the root window
*				 (frame buffer).
*
*    int getRootWindowHeight () - Returns the height of the root window
*				  (frame buffer).
*
*    virtual void  	 setLocation (int, int)       - Sets the location 
*    (X and Y coordinates) of this object.
*
*    virtual void  	 setSize (int, int)           - Sets the size
*    (width and height) of this object.
*
*    virtual void        setX (int)                   - Sets the X coordinate
*    of this object.  Default: Not set.
*
*    virtual void        setY (int)                   - Sets the Y coordinate
*    of this object.  Default: Not set.
*
*    virtual void        setWidth (int)               - Sets the width of this 
*    object.  Default: Not set.
*
*    virtual void 	 setHeight (int)              - Sets the height of this
*    object.  Default: Not set.
*
*    virtual void	 setForegroundColor (Color)   - Finds the associated 
*    index of the Color and sets the foregroundColor to this value. 
*    Default: Not set.
*
*    Color is a UIC color data type. It has the following data members:
*    int red - Contains the red color component,
*    int green - Contains the green color component,
*    int blue - Contains the blue color component.
*
*    virtual void	 setForegroundColor (char *)  - Finds the associated 
*    index of ther named color and sets the foregroundColor to this value. 
*    Default: Not set.
*
*    virtual void	 setBackgroundColor (Color)   -  Finds the associated 
*    index of the Color and sets the backgroundColor to this value. 
*    Default: Not set.
*
*    virtual void	 setBackgroundColor (char *)  - Finds the associated 
*    index of ther named color and sets the backgroundColor to this value. 
*    Default: Not set.
*
*    virtual void	 setEventHandler (UICHandlerFunction) - Sets the 
*    function that is to execute when an event is noted in this subject.  
*    Default: Not set.
*
*    int                 addHotRegion (HotRegion &hot_region)   - Adds a hot 
*    region to the list of hot regions for this object.  Returns the index 
*    of this hot region.
*
*    HotRegion           *removeHotRegion (int index)          -  Removes 
*    the hot region specified by index from the list of hot regions for 
*    this object. 
*
*    HotRegion           *removeHotRegion (HotRegion &hot_region) - Removes
*    the hot region specified by pointer from the list of hot regions for this 
*    object.  
*
*    HotRegion		 *getActiveHotRegion (int index)       - Returns a 
*    pointer to a hot region where the last event took place.  The "index" 
*    is an index with this list.
*
*    Boolean		  isHotRegionEvent ()                  - TRUE if the 
*    last event occurred in a hot region.
*
*    int		  addEventTarget (UIObject&, UICTargetFunction) - Adds
*    an event notification target to the list of targets
*
*    UICGeneric 	  getXViewObject ()          - Returns the XView 
*    pointer of the UIObject.
*
*    void 		  setHelp (char *helpentry)  - Sets the name by which 
*    Help text may be found in the Help file.  Default: Not set.
*
*    virtual int 	  getX ()                    - Returns the current X 
*    coordinate of the object.
*
*    virtual int 	  getY ()                    - Returns the current Y
*    coordinate of the object.
*
*    virtual void	  getLocation (int&, int&)   - Returns the location 
*    (X and Y coordinates) of this object.
*
*    virtual int 	  getHeight ()               - Returns the current 
*    height of the object.
*
*    virtual int 	  getWidth ()                - Returns the current 
*    width of the object.
*
*    virtual void	  getSize (int&, int&)       - Returns the size 
*    (height and width) of this object.
*
*    InputEvent		 *getEvent ()                - Returns the last 
*    event that occurred on this object.
*
*    void                 setEventResult (EventResult result)        -
*    Sets the value that is to be returned to the XView notifier after the 
*    EventHandler function is executed.  Default: NOTIFY_DONE.
*
*    void                 setObjectData (char *name, Generic &data)  -
*    Associates an object with the current one.  Arguments are the name by
*    which the object may be retrieved, and the object itself.
*
*    void                 setObjectData (char *name, Generic *data)  -
*    Associates an object with the current one.  Arguments are the name by
*    which the object may be retrieved, and a pointer to the object.
*
*    Generic*            &getObjectData (char *name)                 - Returns
*    an object that had been previously associated with this one.  The argument
*    is the name by which this object is known.
*
*    Generic             *deleteObjectData (char *name)              - Deletes 
*    the object data corresponding to the specified name.
*
*    Generic             *traverseObjectData (Boolean reset_flag,
*                                             char* &name) -
*    Traverses the object data that you associated with this object.  The
*    member function returns the object data pointer by its return value,
*    and returns the object data name through the char* & argument.  If the
*    Boolean is TRUE, traversal starts over from the beginning of the object
*    chain; otherwise, traversal continues.  When there are no more associated
*    objects, the member function returns NULL.  This function allows you to
*    find out about object data without having to know an object's name.
*
*    void                 setFont (CharacterFont &)   - Sets the font to be
*    used with this object.
*
*    GC                   getFontGC ()                - Gets the X font
*    graphics context of this object.
*
*    void		  consumeEvent (int)          - Event that this object 
*    is interested in knowing about.  The argument is the event ID of the 
*    event.
*
*    void                 useDefaultColorMap ()       - Specifies that the 
*    default color map should be created and used.  If a background or 
*    foreground color is specified and no explicit color map has been set 
*    the default is used automatically.  This results in this member function 
*    not usually being used.
*
*    void                 setColorMap (ColorMap&)     - Sets the ColorMap for
*    this object. Default: Set to default ColorMap.
*
*    virtual void 	  setLabel (char *)           - Sets the label of this 
*    object. Default: Not set.
*
*    virtual char	 *getValue (char *&)          - Virtual function which
*    is implemented in derived classes.
*
*    virtual int	  getValue (int &)            - Virtual function which
*    is implemented in derived classes.
*
*    virtual char	 *getChoice (char *&)         - Virtual function which
*    is implemented in derived classes.
*
*    virtual int 	  getChoice (int &)           - Virtual function which
*    is implemented in derived classes.
*
*    virtual int 	  getChoice (int, int &)      - Virtual function which
*    is implemented in derived classes.
*
*    virtual char	 *getChoice (int, char *&)    - Virtual function which
*    is implemented in derived classes.
*
*    virtual int 	  getNumberSelected ()        - Virtual function which
*    is implemented in derived classes.
*
*    virtual void         show (Boolean)              - Specifies whether or
*    not this object should be displayed.
*
*    virtual Boolean      isBaseWindow ()             - Returns TRUE if the
*    object is a BaseWindow.
*
*    virtual void	  createObject (UIObject *parent=NULL) - Virtual 
*    function which is implemented in derived classes.  createObject is always
*    used to create the XView counterpart to the UIC object.  The parent
*    is the UIC object parent and is used to obtain the XView parent of
*    the XView object.
*
*    void createXViewObjects () - Creates the XView objects corresponding
*    to the main BaseWindow of the application and all of its sibling's XView
*    objects, and any child window XView objects.
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    void        setEventInformation(Event *event)  - Sets the object's
*    internal name information structure.
*
*    void        resetEventInformation()            - Erases or resets the 
*    internal event information.
*
*    void        setHelpData()                      - Sets the internal help 
*    data string.
*
*    PRIVATE MEMBER FUNCTIONS:
*
*    void	 setUIObjectDefaults()              - Sets object's data
*    member's default values.
*
*C-*/
#ifndef _UIOBJECT_H
#define _UIOBJECT_H

// Includes
#include "UICTypes.h"
#include "Generic.h"
#include "GenericHash.h"
#include "GenericList.h"

#ifndef NO_XVPS
#include "postscript_canvas.h"
#endif

// Class Declarations
class ColorMap;
class CharacterFont;
class HotRegion;

// Local Class Definitions
class InputEvent : public Generic {
public:
			 InputEvent () { eventID = UIC_NOT_SET; 
					  eventAction = UIC_NOT_SET; }
			~InputEvent () {} ;

    void                 setXViewEvent (Event *event) { xViewEvent = event; }

    int			 getID ()
			 { return (xViewEvent ? event_id (xViewEvent) :
						UIC_NOT_SET); }
    int			 getAction ()
			 { return (xViewEvent ? event_action (xViewEvent) :
						UIC_NOT_SET); }
    int                  getX ()
			 { return (xViewEvent ? event_x (xViewEvent) :
						UIC_NOT_SET); }
    int                  getY () { return (xViewEvent ? event_y (xViewEvent) :
							UIC_NOT_SET); }
    timeval		*getTime () { return (xViewEvent ?
					      &event_time (xViewEvent) :
					      (timeval *) NULL); }
    Boolean              isButtonUp () { return (xViewEvent ?
						 event_is_up (xViewEvent) :
						 UIC_NOT_SET); }
    Boolean              isButtonDown () { return (xViewEvent ?
						   event_is_down (xViewEvent) :
						   UIC_NOT_SET); }
    Event               *getXViewEvent () { return xViewEvent; }

protected:
    int			  eventID;
    int			  eventAction;
    Boolean               upFlag;
    Boolean               downFlag;
    Event                *xViewEvent;
};

class UITarget : public Generic {
  public:
    UIObject          *targetObject;
    UICTargetFunction  targetFunction;
};

// Class Definition
class UIObject : public Generic {
public:

    			 UIObject ();
			 UIObject (char *);
			~UIObject ();

    void	         initUI (int &argc, char **argv,
			         UICAttribute      attribute = UIC_NO_ATTRIBUTE,
			         UICAttributeValue value = NULL, ...);

    const char          *getVersion ();

    int                  getRootWindowWidth ()
			 { return (rootWindowWidth); }

    int                  getRootWindowHeight ()
			 { return (rootWindowHeight); }

    virtual void  	 setLocation (int, int);
    virtual void  	 setSize (int, int);
    virtual void         setX (int);
    virtual void         setY (int);
    virtual void         setWidth (int);
    virtual void 	 setHeight (int);

    virtual void	 setForegroundColor (Color);
    virtual void	 setForegroundColor (char *);

    virtual void	 setBackgroundColor (Color);
    virtual void	 setBackgroundColor (char *);

    virtual void	 setEventHandler (UICHandlerFunction);

    int			 addHotRegion (HotRegion &hot_region);
    HotRegion           *removeHotRegion (int index);
    HotRegion           *removeHotRegion (HotRegion &hot_region);

    HotRegion		*getActiveHotRegion (int index) 
			 { HotRegion *hot_region = (HotRegion *) NULL;

			   if ((index > 0) &&
			       (index <= activeHotRegions.getSize ()))
			     hot_region = (HotRegion *)
					activeHotRegions [index-1];
			     
			   return hot_region;  }

    Boolean		 isHotRegionEvent ()
			 { return hotRegionEventFlag; }

    int			 addEventTarget (UIObject&, UICTargetFunction);

    UICGeneric 		 getXViewObject ()
			 { return (owObject); }

    void 		 setHelp (char *helpentry)
			 { help = new char [strlen (helpentry) + 1];
    			   strcpy (help, helpentry); }

    virtual int 	 getX ();
    virtual int 	 getY ();
    virtual void	 getLocation (int&, int&);
    virtual int 	 getHeight ();
    virtual int 	 getWidth ();
    virtual void	 getSize (int&, int&);

    InputEvent		*getEvent ()
			 { return (&lastEvent) ; }

    void                 setEventResult (EventResult result)
                         { eventResultSet = TRUE; 
			   eventResult = result; }

    void                 setObjectData (char *name, Generic &data)
                         { objectData->addItem (name, &data); }

    void                 setObjectData (char *name, Generic *data)
                         { objectData->addItem (name, data); }

    Generic*            &getObjectData (char *name)
                         { return (*objectData) [name]; }

    Generic             *deleteObjectData (char *name)
			 { return objectData->removeItem (name); }

    Generic             *traverseObjectData (Boolean reset_flag, char* &name)
                         { return objectData->traverse (reset_flag, name); }

    void                 setFont (CharacterFont &);
    GC                   getFontGC ();
    void		 consumeEvent (int);
    void                 useDefaultColorMap ();
    void                 setColorMap (ColorMap&);

    virtual void 	 setLabel (char *);
    virtual char	*getValue (char *&);
    virtual int		 getValue (int &);
    virtual char	*getChoice (char *&);
    virtual int 	 getChoice (int &);
    virtual int 	 getChoice (int, int &);
    virtual char	*getChoice (int, char *&);
    virtual int 	 getNumberSelected ();

    virtual void         show (Boolean);
    virtual Boolean      isBaseWindow ();
    virtual void	 createObject (UIObject *parent=NULL);

    void                 createXViewObjects ()
			 { if (!baseWindow->getXViewObject ())
			     ((UIObject *) baseWindow)->createObject (); }

protected:
    int 	 	 xLoc;
    int		 	 yLoc;
    int	 	 	 height;
    int 	 	 width;
    char		*label;
    char 		*help;
    UICGeneric	 	 owObject;
    int			 foregroundColor;
    int			 backgroundColor;
    UICHandlerFunction	 eventHandler;
    GenericList		 events;
    InputEvent		 lastEvent;
    Boolean		 eventResultSet;
    EventResult		 eventResult;
    GenericList		 hotRegions;
    GenericList		 activeHotRegions;
    Boolean		 hotRegionEventFlag;
    GenericList		 eventTargets;
    CharacterFont       *font;
    GC                   fontGC;
    GenericHash         *objectData; 
    Boolean		 showFlag;
    static Display      *rootDisplay;
    static int		 rootWindowWidth;
    static int		 rootWindowHeight;
    static ColorMap	*colorMap;
    static Boolean       defaultColorMap;
    static UIObject	*baseWindow;
    static int           numUIObjects;
    static int           totalUIObjectsCreated;
    int			 uiObjectID;

    void                 setEventInformation (Event *event);
    void                 resetEventInformation ();

    void                 setHelpData ()
      { if (help) xv_set (owObject, XV_HELP_DATA, help, NULL); }

private:
    void		 setUIObjectDefaults ();
};

#endif /* _UIOBJECT_H */
