/*H+ guic.c
*
*    SCCS ID: @(#)guic.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Convert a GUIDE intermediate language (GIL) file to UIC source files.
*    The following files are created:
*
*	<app>_ui.cc		Code to initialize user interface objects.
*				This file will be compiled and linked with
*				the application.
*
*	<app>_stubs.cc		Callback function stubs.  These are example
*				handler functions that will serve as a template
*				for the actual handler function in the
*				application.
*
*	<app>.info		Help file.  This file contains object help
*				text indexed by the object name.
*
*	Makefile		Makefile that can be used to compile and run
*				unmodified GUIDE applications.  This file is
*				generated only if there is no makefile in the
*				current directory.
*
*    GLOBAL ROUTINES:
*
*    guic_application - Gets the name of the application.
*    guic_application_file - Gets the name of the application file.
*    guic_boolean_string - Returns a "TRUE" or "FALSE" string.
*    guic_calloc - Calloc replacement function.
*    guic_error - Outputs guic error.
*    guic_gil_file - Gets the application GIL (.G) file name.
*    guic_identifier_ok - Determines if string is a legal identifier.
*    guic_iscontrol - Returns TRUE if the object is a control (UIComponent).
*    guic_ispane - Returns TRUE if the object is a pane (UIDisplay).
*    guic_iswindow - Returns TRUE if the object is a window (UIWindow).
*    guic_malloc - Malloc replacement function.
*    guic_program - Returns the name of the program being executed
*                   (usually "guic").
*    guic_realloc - Realloc replacement function.
*    guic_stub_written - Determines is a stub has been written or not.
*    guic_silent - Returns TRUE if running in silent mode.
*    guic_verbose - Returns TRUE if running in silent mode.
*    guic_medium_names - Returns TRUE if medium length names are to be
*                        generated (parent_child).
*    guic_long_names - Returns TRUE if long names are to be generated
*                      (..._grandparent_parent_child).
*
*    LOCAL ROUTINES:
*
*    main - The main function.
*    set_application - Determines and sets the base name of the GIL (.G)
*                      file being processed.
*    set_program - Determines and sets the base name of the program
*                  being executed: guic.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)guic.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <sys/param.h>
#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include <guide.h>
#include "memlib.h"
#include "guic.h"
#include "init.h"
#include "load.h"
#include "stubs.h"
#include "help.h"

/* Global Variables */
static char	application_g [MAXPATHLEN];	/* application name */
static char    *program_g;			/* program name */
static int	silent_g = FALSE;		/* true if in silent mode */
static int	verbose_g = FALSE;		/* true if in verbose mode */
static int      medium_names_g = FALSE;         /* true if object names are to
						   be generated using the
						   parent of the object */
static int      long_names_g = FALSE;           /* true if object names are to
						   be generated using the
						   entire list of owners of
						   objects going back to
						   the base frame. */

/* Local Function Declarations */
void		set_application();
void		set_program();

/*F+ main (int argc, char **argv, char **envp)
*
*    PURPOSE:
*
*    The main function.
*
*    PARAMETERS:
*
*    argc - Number of args.
*    argv - The arguments.
*    envp - Environment variables.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
main(argc, argv, envp)
	int             argc;
	char          **argv;
	char          **envp;
{
	extern int	optind;			/* option index */
	extern char    *optarg;			/* option argument */
	int		error = FALSE;		/* TRUE if bad usage */
	int		ch;			/* work variable */

	set_program(argv[0]);
	while ((ch = getopt(argc, argv, "svlm")) != -1) {
		switch (ch) {
		case 'v':
			if (silent_g)
				error = TRUE;
			else
				verbose_g = TRUE;
			break;

		case 's':
			if (verbose_g)
				error = TRUE;
			else
				silent_g = TRUE;
			break;

		case 'l':
			if (medium_names_g)
			  error = TRUE;
			else
			  long_names_g = TRUE;
			break;

		case 'm':
			if (long_names_g)
			  error = TRUE;
			else
			  medium_names_g = TRUE;
			break;

		default:
			error = TRUE;
		}
	}

	if (optind == argc - 1) 
		set_application(argv[optind]);
	else
		error = TRUE;

	if (error) {
		fprintf(stderr,
		        "Usage: %s [-s|-v][-m|-l] interface\n",
			program_g);
		exit(1);
	}

	if (!guic_identifier_ok(guic_application())) {
		fprintf(stderr,
			"%s: interface filename must be a valid C identifier\n",
			program_g);
		exit(1);
	}

	/*
	 * Read the input file and create the various output files.
	 */
	if (guic_load_file() == ERROR ||
	    guic_write_init_file() == ERROR ||
	    guic_write_stubs_file() == ERROR ||
	    guic_write_help_file() == ERROR ||
	    guic_write_make_file() == ERROR)
		exit(1);

	exit(0);
}


/*
 * Functions that implement GUIDE's naming conventions.
 */

/*F+ char *guic_gil_file ()
*
*    PURPOSE:
*
*    Return the application GIL file.
*
*    RETURN VALUES:
*
*    Returns the name of the GIL file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_gil_file()
{
	static char	name[MAXPATHLEN];

	if (!*name) {
		strcpy(name, guic_application_file());
		gio_expand_gil_path(name);
	}
	return name;
}


/*
 * Miscellaneous utility functions.
 */

/*F+ char *guic_application ()
*
*    PURPOSE:
*
*    Return the application name.
*
*    RETURN VALUES:
*
*    Returns the name of the application (base name of GIL (.G) file).
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_application()
{
	char	       *p = rindex(application_g, '/');

	return p ? p + 1 : application_g;
}

/*F+ char *guic_application_file ()
*
*    PURPOSE:
*
*    Return the application file name.
*
*    RETURN VALUES:
*
*    Returns the name of the application file (GIL file name without .G).
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_application_file()
{
	return application_g;
}

/*F+ char *guic_boolean_string (int i)
*
*    PURPOSE:
*
*    Return the printed representation of a boolean.
*
*    PARAMETERS:
*
*    i - If 0 return "FALSE", otherwise, return "TRUE".
*
*    RETURN VALUES:
*
*    Returns "TRUE" or "FALSE".
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_boolean_string(i)
	int		i;
{
	return i ? "TRUE" : "FALSE";
}

/*F+ void guic_error (char *s)
*
*    PURPOSE:
*
*    Output a guic error message.
*
*    PARAMETERS:
*
*    s - The message to output.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
void
guic_error(s)
	char           *s;
{
	fprintf(stderr, "%s: %s\n", guic_program(), s);
}

/*F+ int guic_identifier_ok (char *s)
*
*    PURPOSE:
*
*    Check a string to see if contains a legal C++ identifier.
*
*    PARAMETERS:
*
*    s - The identifier to check.
*
*    RETURN VALUES:
*
*    Returns FALSE if the idenetifier is illegal.  Returns TRUE if the
*    identifier is legal.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_identifier_ok(s)
	char	*s;
{
	if(s && !isalpha(*s++))
		return FALSE;

	while(s && *s != '\0') {
		if(!isalnum(*s) && (*s != '_')) {
			return FALSE;
		}
		s++;
	}

	return TRUE;
}

/*F+ int guic_iscontrol (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Return whether a type is a control (UIComponent).
*
*    PARAMETERS:
*
*    obj - The object to check.
*
*    RETURN VALUES:
*
*    Returns FALSE if the object is not a control object.  Returns TRUE
*    if the object is a control object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_iscontrol(obj)
	GUIC_OBJ	       *obj;
{
	return (obj->type == G_BUTTON ||
		obj->type == G_GAUGE ||
		obj->type == G_MESSAGE ||
		obj->type == G_SCROLLING_LIST ||
		obj->type == G_SETTING ||
		obj->type == G_SLIDER ||
		obj->type == G_TEXT_FIELD);
}

/*F+ int guic_ispane (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Return whether an object is a pane (UIDisplay).
*
*    PARAMETERS:
*
*    obj - The object to check.
*
*    RETURN VALUES:
*
*    Returns FALSE if the object is not a pane object.  Returns TRUE
*    if the object is a pane object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_ispane(obj)
	GUIC_OBJ	       *obj;
{
	return (obj->type == G_CANVAS_PANE ||
		obj->type == G_CONTROL_AREA ||
		obj->type == G_TEXT_PANE ||
		obj->type == G_TERM_PANE);
}

/*F+ int guic_iswindow (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Return whether an object is a window.
*
*    PARAMETERS:
*
*    obj - The object to check.
*
*    RETURN VALUES:
*
*    Returns FALSE if the object is not a window object.  Returns TRUE
*    if the object is a window object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_iswindow(obj)
	GUIC_OBJ	       *obj;
{
	return (obj->type == G_BASE_WINDOW ||
		obj->type == G_POPUP_WINDOW);
}

/*F+ char *guic_program ()
*
*    PURPOSE:
*
*    Return the name of the program being executed (usually "guic").
*
*    RETURN VALUES:
*
*    Returns the name of the program.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_program()
{
	return program_g;
}

/*F+ int guic_silent ()
*
*    PURPOSE:
*
*    Return if in silent mode.
*
*    RETURN VALUES:
*
*    Returns FALSE if not silent mode.  Returns TRUE if in silent mode.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_silent()
{
	return silent_g;
}

/*F+ int guic_verbose ()
*
*    PURPOSE:
*
*    Return if in verbose mode.
*
*    RETURN VALUES:
*
*    Returns FALSE if not verbose mode.  Returns TRUE if in verbose mode.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_verbose()
{
	return verbose_g;
}

/*F+ int guic_long_names ()
*
*    PURPOSE:
*
*    Return if long names are to be generated (...grandparent_parent_child).
*
*    RETURN VALUES:
*
*    Returns FALSE if long names are not to be generated.  Returns TRUE
*    if long names are to be generated.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int guic_long_names ()
{
  return long_names_g;
}

/*F+ int guic_medium_names ()
*
*    PURPOSE:
*
*    Return if medium length names are to be generated (parent_child).
*
*    RETURN VALUES:
*
*    Returns FALSE if medium length names are not to be generated.  Returns
*    TRUE if medium length names are to be generated.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int guic_medium_names ()
{
  return medium_names_g;
}


/*
 * Memory allocation functions.
 */

/*F+ char *calloc (int n, int size)
*
*    PURPOSE:
*
*    Version of calloc() with error checking.
*
*    PARAMETERS:
*
*    n    - Number of memory chunks to allocate.
*    size - Size of memory chunks.
*
*    RETURN VALUES:
*
*    Returns pointer to allocated memory.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char           *
guic_calloc(n, size)
	unsigned        n;
	unsigned        size;
{
	char           *p;
	extern char    *calloc();

	if (p = CALLOC (n, size))
		return p;
	guic_error("calloc: out of memory");
	/*NOTREACHED*/
}

/*F+ char *malloc (int size)
*
*    PURPOSE:
*
*    Version of malloc() with error checking.
*
*    PARAMETERS:
*
*    size - Size of memory to allocate.
*
*    RETURN VALUES:
*
*    Returns pointer to allocated memory.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char           *
guic_malloc(size)
	unsigned        size;
{
	char           *p;

	if (p = MALLOC (size))
		return p;
	guic_error("malloc: out of memory");
	/*NOTREACHED*/
}

/*F+ char *realloc (char *buf, int size)
*
*    PURPOSE:
*
*    Version of realloc() with error checking.
*
*    PARAMETERS:
*
*    buf  - Pointer to memory to reallocate.
*    size - Size of memory to allocate.
*
*    RETURN VALUES:
*
*    Returns pointer to allocated memory.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char           *
guic_realloc(buf, size)
	char           *buf;
	unsigned        size;
{
	char           *p;

	if (p = REALLOC (buf, size))
		return p;
	guic_error("realloc: out of memory");
	/*NOTREACHED*/
}

/*F+ int guic_stub_written (char *name)
*
*    PURPOSE:
*
*    Return whether a stub has already been written.  If not, add it
*    to the list of those already written.
*
*    PARAMETERS:
*
*    name - The name of the stub to check for.
*
*    RETURN VALUES:
*
*    Returns FALSE if the stub doesn't exist, else return TRUE.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int guic_stub_written (name)
	char	       *name;
{
#define	INC	10				/* table buffer increment */
#define	SIZE	(sizeof (char **))		/* table element size */

	static char	**tab;
	static int	lth = 0;		/* table size */
	static int	c = 0;			/* table count */
	int		i;

	/*
	 * See if the function is in the table.
	 */
	for (i = 0; i < c; i++)
		if (strcmp(tab[i], name) == 0)
			return TRUE;

	/*
	 * Add the function to the table.
	 */
	if (c == lth)
		tab = tab ?
			(char **) guic_realloc(tab, (lth += INC) * SIZE) :
			(char **) guic_malloc((lth += INC) * SIZE);
	tab[c++] = name;
	return FALSE;

#undef INC
#undef SIZE
}

/*F+ static void set_application ()
*
*    PURPOSE:
*
*    Set the GUIDE application name (full path name of the GIL file
*    without the .G).
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
set_application(name)
	char	       *name;
{
	char	       *p;

	strcpy(application_g, name);

	/*
	 * Kludge: expand to GIL file name then remove GIL suffix.
	 */
	gio_expand_gil_path(application_g);
	if (p = rindex(application_g, '.'))
		*p = '\0';
}

/*F+ static void set_program ()
*
*    PURPOSE:
*
*    Set the program name (usually "guic").
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
set_program(name)
	char	       *name;
{
	char	       *p = rindex(name, '/');

	program_g = (p ? p + 1 : name);
}

#ifdef TEST
#endif
