/*H+ load.c
*
*    SCCS ID: @(#)load.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Routines to load in GUIDE intermediate language (GIL) files.
*
*    GLOBAL ROUTINES:
*
*    guic_load_file - Load the application GIL file into memory.
*
*    LOCAL ROUTINES:
*
*    loadmsg - Build a status message for the current load operation.
*    load_attribute - Load one attribute.
*    load_booleans - Load an array of booleans.
*    load_events - Load an array of event types.
*    load_label_types - Load an array of label types.
*    load_handlers - Load a NULL-terminated array of handler names.
*    load_names - Load a NULL-terminated array of names.
*    load_object - Load one object.
*    load_strings - Load a NULL-terminated array of strings.
*    set_attribute - Set the current attribute for use in status messages.
*    set_object - Set the current object for use in status messages.
*    guic_get_string - Gets a single string and substitutes \" for a ".
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)load.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <sys/types.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "load.h"
#include "list.h"

/* Macro Definitions */

/*
 * Error messages.
 */
#define	ERR_EOF			"unexpected end-of-file"
#define	ERR_FILE_FORMAT		"unknown file format"
#define	ERR_UNKNOWN		"unknown value"
#define ERR_UNKNOWN_OBJECT	"unknown object"
#define	ERR_WANT_INTEGER	"cannot find integer"
#define	ERR_WANT_LIST		"cannot find start of list"
#define	ERR_WANT_NAME		"cannot find start of name"
#define	ERR_WANT_OBJECT		"cannot find start of object"
#define	ERR_WANT_STRING		"cannot find string"
#define ERR_WANT_BOOLEAN	"cannot find boolean"
#define ERR_WANT_KEYWORD	"cannot find keyword"
#define ERR_BAD_IDENTIFIER	"not a valid C identifier"

/*
 * Warnings.
 */
#define WARN_BAD_HANDLER	"WARNING not a C identifier, ignored"

/* Global Variables */

/*
 * Miscellaneous global variables.
 */
static char     Attribute[64];			/* current attribute */
static char     Object[64];			/* current object */

/*
 * Internal procedure declarations.
 */
char           *loadmsg ();
char           *load_attribute ();
char	       *load_booleans ();
char	       *load_events ();
char	       *load_label_types ();
char	       *load_handlers ();
char	       *load_names ();
char	       *load_object ();
char	       *load_strings ();
void            set_attribute ();
void            set_object ();
int		guic_get_string ();

/*F+ int guic_load_file ()
*
*    PURPOSE:
*
*    Read a GIL file into memory.
*
*    RETURN VALUES:
*
*    Returns OK if successful.  Returns ERROR if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_load_file()
{
	char           *errmsg;

	/*
	 * Open the input file.
	 */
	if (errmsg = gio_open_gil_input(guic_gil_file())) {
		guic_error(errmsg);
		return ERROR;
	}
	if (!guic_silent())
		fprintf(stderr, "%s: reading %s\n", guic_program(),
			guic_gil_file());

	/*
	 * Read the file header.
	 */
	if (!gio_get_file_begin()) {
		guic_error(loadmsg(ERR_FILE_FORMAT));
		return ERROR;
	}

	/*
	 * Loop reading the input file until there are no more objects.
	 */
	while (!gio_get_file_end()) {
		if (gio_get_eof()) {
			guic_error(loadmsg(ERR_EOF));
			return ERROR;
		}

		if (!gio_get_object_begin()) {
			guic_error(loadmsg(ERR_WANT_OBJECT));
			return ERROR;
		}

		set_object("");
		if (errmsg = load_object()) {
			guic_error(errmsg);
			return ERROR;
		}
	}

	return OK;
}

/*F+ static char *load_object ()
*
*    PURPOSE:
*
*    Load one GIL object.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static char       *
load_object()
{
	GUIC_OBJ	       *obj = (GUIC_OBJ *) guic_calloc(1, sizeof (GUIC_OBJ));
	char           *errmsg;
	char	       *keyword;

	for (;;) {
		if (gio_get_eof()) {
			errmsg = loadmsg(ERR_EOF);
			break;
		}
		if (gio_get_object_end()) {
			guic_append(obj);
			return NULL;
		}
		if (!gio_get_keyword(&keyword)) {
			errmsg = loadmsg(ERR_WANT_KEYWORD);
			break;
		}

		set_attribute(keyword);
		if (errmsg = load_attribute(obj, G_string_to_attr(keyword)))
			break;
		set_attribute(NULL);
	}

	return errmsg;
}

/*F+ static char *load_attribute ()
*
*    PURPOSE:
*
*    Load one GIL object attribute.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static char    *
load_attribute(obj, attr)
	GUIC_OBJ        *obj;
	G_ATTRS         attr;
{
	int		i;
	char	       *s;
	char	       *errmsg;

	if (gio_get_eof())
		return loadmsg(ERR_EOF);

	switch (attr) {
	case G_BACKGROUND_COLOR:
		if (!guic_get_string(&obj->background_color))
			return loadmsg(ERR_WANT_STRING);
		if (obj->background_color) {
			if (guic_iscontrol(obj) && obj->owner)
				obj->owner->color_needed = TRUE;
			if (guic_iswindow(obj) || guic_ispane(obj))
				obj->color_needed = TRUE;
		}
		break;
		
	case G_BUTTON_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->button_type = G_string_to_button_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;

	case G_CHOICES:
		if (errmsg = load_strings(&obj->choices))
			return errmsg;
		break;
		
	case G_CHOICE_LABEL_TYPES:
		if (errmsg = load_label_types(&obj->choice_label_type))
			return errmsg;
		break;
		
	case G_CHOICE_COLORS:
		if (errmsg = load_strings(&obj->choice_color))
			return errmsg;
		break;

	case G_COLUMNS:
		if (!gio_get_integer(&obj->columns))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_CONSTANT_WIDTH:
		if (!gio_get_boolean(&obj->constant_width))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_DONE_HANDLER:
		if (!gio_get_handler(&obj->done_handler))
			return loadmsg(ERR_WANT_NAME);

		if (!guic_identifier_ok(obj->done_handler)) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free(obj->done_handler);
			obj->done_handler = NULL;
		}
		break;

	case G_DRAWING_MODEL:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->drawing_model = G_string_to_drawing_model(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_EVENTS:
		if (errmsg = load_events(&obj->events))
			return errmsg;
		break;
		
	case G_EVENT_HANDLER:
		if (!gio_get_handler(&obj->event_handler))
			return loadmsg(ERR_WANT_NAME);

		if (!guic_identifier_ok(obj->event_handler)) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free(obj->event_handler);
			obj->event_handler = NULL;
		}
		break;
		
	case G_FOREGROUND_COLOR:
		if (!guic_get_string(&obj->foreground_color))
			return loadmsg(ERR_WANT_STRING);
		if (obj->foreground_color) {
			if (guic_iscontrol(obj) && obj->owner)
				obj->owner->color_needed = TRUE;
			if (guic_iswindow(obj) || guic_ispane(obj))
				obj->color_needed = TRUE;
		}
		break;
		
	case G_HEIGHT:
		if (!gio_get_integer(&obj->height))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_HELP:
		if (!guic_get_string(&obj->help))
			return loadmsg(ERR_WANT_STRING);
		break;
		
	case G_HSCROLL:
		if (!gio_get_boolean(&obj->hscrollbar))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_ICON:
		if (!guic_get_string(&obj->icon))
			return loadmsg(ERR_WANT_STRING);
		break;
		
	case G_ICON_MASK:
		if (!guic_get_string(&obj->icon_mask))
			return loadmsg(ERR_WANT_STRING);
		break;
		
	case G_LABEL:
		if (!guic_get_string(&obj->label))
			return loadmsg(ERR_WANT_STRING);
		break;
		
	case G_LABEL_BOLD:
		if (!gio_get_boolean(&obj->label_bold))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_LABEL_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->label_type = G_string_to_label_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_LAYOUT_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->layout_type = G_string_to_layout_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_MAPPED:
		if (!gio_get_boolean(&obj->mapped))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_MAX_VALUE:
		if (!gio_get_integer(&obj->max_value))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_MENU_HANDLER:
		if (!gio_get_handler(&obj->menu_handler))
			return loadmsg(ERR_WANT_NAME);

		if (!guic_identifier_ok(obj->menu_handler)) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free(obj->menu_handler);
			obj->menu_handler = NULL;
		}
		break;
		
	case G_MENU_ITEM_COLORS:
		if (errmsg = load_strings(&obj->menu_item_color))
			return errmsg;
		break;

	case G_MENU_ITEM_DEFAULTS:
		if (errmsg = load_booleans(&obj->menu_item_default))
			return errmsg;
		break;
		
	case G_MENU_ITEM_HANDLERS:
		if (errmsg = load_handlers(&obj->menu_item_handler))
			return errmsg;
		break;
		
	case G_MENU_ITEM_LABELS:
		if (errmsg = load_strings(&obj->menu_item_label))
			return errmsg;
		break;
		
	case G_MENU_ITEM_LABEL_TYPES:
		if (errmsg = load_label_types(&obj->menu_item_label_type))
			return errmsg;
		break;
		
	case G_MENU_ITEM_MENUS:
		if (errmsg = load_names(&obj->menu_item_menu))
			return errmsg;
		break;
		
	case G_MENU_NAME:
		if (!gio_get_name(&obj->menu_name))
			return loadmsg(ERR_WANT_NAME);
		if (!guic_identifier_ok(obj->menu_name))
			return loadmsg(ERR_BAD_IDENTIFIER);
		break;
		
	case G_MENU_TITLE:
		if (!guic_get_string(&obj->menu_title))
			return loadmsg(ERR_WANT_STRING);
		break;
		
	case G_MENU_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->menu_type = G_string_to_menu_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_MIN_VALUE:
		if (!gio_get_integer(&obj->min_value))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_MULTIPLE_SELECTIONS:
		if (!gio_get_boolean(&obj->multiple_selections))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_NAME:
		if (!gio_get_name(&obj->name))
			return loadmsg(ERR_WANT_NAME);
		if (!guic_identifier_ok(obj->name))
			return loadmsg(ERR_BAD_IDENTIFIER);
		set_object(obj->name);
		break;
		
	case G_NOTIFY_HANDLER:
		if (!gio_get_handler(&obj->notify_handler))
			return loadmsg(ERR_WANT_NAME);

		if (!guic_identifier_ok(obj->notify_handler)) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free(obj->notify_handler);
			obj->notify_handler = NULL;
		}
		break;
		
	case G_ORIENTATION:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->orientation = G_string_to_layout_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_OWNER:
		if (!gio_get_name(&s))
			return loadmsg(ERR_WANT_NAME);
		if (s && ((obj->owner = guic_object(s)) == NULL))
			return loadmsg(ERR_UNKNOWN_OBJECT);
		if (!guic_identifier_ok(s))
			return loadmsg(ERR_BAD_IDENTIFIER);
		break;
		
	case G_PINNABLE:
		if (!gio_get_boolean(&obj->pinnable))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_PINNED:
		if (!gio_get_boolean(&obj->pinned))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_READ_ONLY:
		if (!gio_get_boolean(&obj->read_only))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_REPAINT_PROC:
		if (!gio_get_handler(&obj->repaint_proc))
			return loadmsg(ERR_WANT_NAME);

		if (!guic_identifier_ok(obj->repaint_proc)) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free(obj->repaint_proc);
			obj->repaint_proc = NULL;
		}
		break;
		
	case G_RESIZABLE:
		if (!gio_get_boolean(&obj->resizable))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_ROWS:
		if (!gio_get_integer(&obj->rows))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_SCROLLABLE_HEIGHT:
		if (!gio_get_integer(&obj->scrollable_height))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_SCROLLABLE_WIDTH:
		if (!gio_get_integer(&obj->scrollable_width))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_SELECTION_REQUIRED:
		if (!gio_get_boolean(&obj->selection_required))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SETTING_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->setting_type = G_string_to_setting_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_SHOW_BORDER:
		if (!gio_get_boolean(&obj->show_border))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SHOW_ENDBOXES:
		if (!gio_get_boolean(&obj->show_endboxes))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SHOW_FOOTER:
		if (!gio_get_boolean(&obj->show_footer))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SHOW_RANGE:
		if (!gio_get_boolean(&obj->show_range))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SHOW_VALUE:
		if (!gio_get_boolean(&obj->show_value))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_SLIDER_WIDTH:
		if (!gio_get_integer(&obj->slider_width))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_STORED_LENGTH:
		if (!gio_get_integer(&obj->stored_length))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_TEXT_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->text_type = G_string_to_text_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		break;
		
	case G_TICKS:
		if (!gio_get_integer(&obj->ticks))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_TYPE:
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		if ((obj->type = G_string_to_type(s)) == ERROR)
			return loadmsg(ERR_UNKNOWN);
		set_object(s);
		break;

	case G_USER_DATA:
		if (!gio_get_list(&s))
			return loadmsg(ERR_WANT_LIST);
		free(s);			/* ignore */
		break;

	case G_VALUE_LENGTH:
		if (!gio_get_integer(&obj->value_length))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_VALUE_X:
		if (!gio_get_integer(&obj->value_x))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_VALUE_Y:
		if (!gio_get_integer(&obj->value_y))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_VSCROLL:
		if (!gio_get_boolean(&obj->vscrollbar))
			return loadmsg(ERR_WANT_BOOLEAN);
		break;
		
	case G_WIDTH:
		if (!gio_get_integer(&obj->width))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_X:
		if (!gio_get_integer(&obj->x))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	case G_Y:
		if (!gio_get_integer(&obj->y))
			return loadmsg(ERR_WANT_INTEGER);
		break;
		
	default:
		return loadmsg(ERR_UNKNOWN);
	}

	return NULL;
}

/*
 * Build a status message including the current object, the current
 * attribute, and the specified message.
 */
static char    *
loadmsg(s)
	char           *s;
{
	static char     msg[256];

	if (*Object && *Attribute)
		sprintf(msg, "object %s: attribute %s: %s",
			Object, Attribute, s);

	else if (*Attribute)
		sprintf(msg, "attribute %s: %s", Attribute, s);

	else if (*Object)
		sprintf(msg, "object %s: %s", Object, s);

	else
		strcpy(msg, s);

	return msg;
}

/*F+ static char *load_booleans (int **p)
*
*    PURPOSE:
*
*    Load an array of booleans from the input file into an array terminated
*    by -1.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_booleans(p)
	int		**p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (int *))		/* array element size */
	int             lth = 0;		/* array length */
	int             c;			/* array count */

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (int *) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (int *) guic_realloc(*p, (lth += INC) * SIZE);
		if (!gio_get_boolean(&(*p)[c]))
			return loadmsg(ERR_WANT_BOOLEAN);
	}

	(*p)[c] = -1;
	return NULL;

#undef	INC
#undef	SIZE
}

/*F+ static char *load_events (int **p)
*
*    PURPOSE:
*
*    Load an array of event types from the input file into an array terminated
*    by -1.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_events(p)
	G_EVENT_TYPES	**p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (G_EVENT_TYPES *))	/* array element size */
	int		lth = 0;		/* array length */
	int		c;			/* array count */
	char	       *s;			/* work string */

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (G_EVENT_TYPES *) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (G_EVENT_TYPES *) guic_realloc(*p, (lth += INC) * SIZE);
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		(*p)[c] = G_string_to_event_type(s);
	}

	(*p)[c] = -1;
	return NULL;
#undef	INC
#undef	SIZE
}

/*F+ static char *load_handlers (char ***p)
*
*    PURPOSE:
*
*    Load a null-terminated array of handler names from the input file into
*    an array.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_handlers(p)
	char         ***p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (char **))		/* array element size */
	int             lth = 0;		/* array length */
	int             c;			/* array count */

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (char **) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (char **) guic_realloc(*p, (lth += INC) * SIZE);
		if (!gio_get_handler(&(*p)[c]))
			return loadmsg(ERR_WANT_STRING);

		if (!guic_identifier_ok((*p)[c])) {
			guic_error(loadmsg(WARN_BAD_HANDLER));
			free((*p)[c]);
			(*p)[c] = NULL;
		}
	}

	(*p)[c] = NULL;
	return NULL;
#undef	INC
#undef	SIZE
}

/*F+ static char *load_names (char ***p)
*
*    PURPOSE:
*
*    Load a null-terminated array of names from the input file into an array.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_names(p)
	char         ***p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (char **))		/* array element size */
	int             lth = 0;		/* array length */
	int             c;			/* array count */

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (char **) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (char **) guic_realloc(*p, (lth += INC) * SIZE);
		if (!gio_get_name(&(*p)[c]))
			return loadmsg(ERR_WANT_STRING);
		if (!guic_identifier_ok((*p)[c]))
			return loadmsg(ERR_BAD_IDENTIFIER);
	}

	(*p)[c] = NULL;
	return NULL;
#undef	INC
#undef	SIZE
}

/*F+ static char *load_label_types (G_LABEL_TYPE **p)
*
*    PURPOSE:
*
*    Load an array of label types from the input file into an array.
*    Terminate the array with -1.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_label_types(p)
	G_LABEL_TYPES	**p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (G_LABEL_TYPES *))	/* array element size */
	int		lth = 0;		/* array length */
	int		c;			/* array count */
	char	       *s;			/* work string */

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (G_LABEL_TYPES *) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (G_LABEL_TYPES *) guic_realloc(*p, (lth += INC) * SIZE);
		if (!gio_get_keyword(&s))
			return loadmsg(ERR_WANT_KEYWORD);
		(*p)[c] = G_string_to_label_type(s);
	}

	(*p)[c] = -1;
	return NULL;
#undef	INC
#undef	SIZE
}

/*F+ static char *load_strings (char ***p)
*
*    PURPOSE:
*
*    Load a null-terminated array of strings from the input file into an array.
*    NULL strings are turned into empty strings instead of being left NULL.
*
*    PARAMETERS:
*
*    p - The array to load.
*
*    RETURN VALUES:
*
*    Returns NULL if successful.  Returns an error message if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
load_strings(p)
	char         ***p;
{
#define	INC	10				/* array buffer increment */
#define	SIZE	(sizeof (char **))		/* array element size */
	int             lth = 0;		/* array length */
	int             c;			/* array count */
	char	       *s;

	if (!gio_get_list_begin())
		return loadmsg(ERR_WANT_LIST);

	*p = (char **) guic_malloc((lth += INC) * SIZE);

	for (c = 0; !gio_get_list_end(); c++) {
		if (c + 1 == lth)
			*p = (char **) guic_realloc(*p, (lth += INC) * SIZE);
		if (!guic_get_string(&s))
			return loadmsg(ERR_WANT_STRING);
		(*p)[c] = s ? s : "";
	}

	(*p)[c] = NULL;
	return NULL;
#undef	INC
#undef	SIZE
}

/*F+ static void set_attribute (char *s)
*
*    PURPOSE:
*
*    Set the current attribute for use in status/error messages.
*
*    PARAMETERS:
*
*    s - The name of the attribute.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
set_attribute(s)
	char           *s;
{
	strcpy(Attribute, s ? s : "");
}

/*F+ static void set_object (char *s)
*
*    PURPOSE:
*
*    Set the current object for use in status/error messages.
*
*    PARAMETERS:
*
*    s - The name of the object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
set_object(s)
	char           *s;
{
	strcpy(Object, s ? s : "");
}

/*F+ static int guic_get_string (char **s)
*
*    PURPOSE:
*
*    Loads a string from the GIL file.  If there is a " in the
*    string, it is substituted with a \".
*
*    PARAMETERS:
*
*    s - The string.
*
*    RETURN VALUES:
*
*    Returns non-zero if a string was loaded.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static int guic_get_string (s)
char **s;
{
  int   i = 0;
  int   num_quotes = 0;
  char *pos = (char *) NULL;
  int   result;
  char *string;

  /* Get the string from the GIL file */
  if (result = gio_get_string (s)) {
    if (*s) {
      pos = strchr (*s, '\"');
 
      /* Count the quotes in the string */
      while (pos) {
        num_quotes++;
        pos = strchr (pos+1, '\"');
      }     

      /* If there were quotes... */
      if (num_quotes) {
        string = guic_malloc (strlen (*s) + num_quotes + 1);

        /* Copy the string with substitution of \" for " */
        pos = *s;
        while (*pos) {
          if (*pos == '\"')
            string [i++] = '\\';

	  string [i++] = *pos++;
        }
        string [i] = '\0';

        /* Make sure the new string is returned */
        *s = string;
      }
    }
  }

  return (result);
}

#ifdef TEST
#endif
