/**H+memlib.c ***************************************************************
*
*    SCCS ID: @(#)memlib.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Replacement functions for the standard memory management functions
*    provided in the C run-time library.
*
*    The following describes each function provided and how the
*    function should be called:
*
*    mem_Calloc:
*
*         Use: CALLOC (n, s)
*
*         Returns a pointer to memory filled with 0's.  The length
*         of the memory returned is n * s.  If n * s <= 0 then
*         NULL is returned.
*
*    mem_Malloc:
*
*         Use: MALLOC (s)
*
*         Returns a pointer to s bytes of memory.  If s <= 0 then
*         NULL is returned.
*
*    mem_Realloc:
*
*         Use: REALLOC (m, s)
*
*         Returns a pointer to s bytes of memory that contains as much
*         of the memory pointed to by m that will fit.  If s <= 0 then
*         m is returned unchanged.  If m is NULL then REALLOC acts
*         like MALLOC.
*
*    mem_Free:
*
*         Use: FREE (m)
*
*         Frees up the memory pointed to by m.  If m is NULL, nothing
*         happens.  If an invalid pointer is passed to FREE while in
*         MEMORY_MANAGEMENT mode (see below) then information about
*         where the call to free was made is output and a core dump
*         is caused.
*
*    mem_FreeNull:
*
*         Use: FREE_NULL (m)
*
*         Does the same thing as FREE except that NULL is always returned.
*         Since NULL is always returned, a pointer can be freed and
*         initialized at the same time:
*
*                            m = FREE_NULL (m);
*
*    mem_SetCheck:
*
*         Use: mem_SetCheck (flag)
*
*         If in MEMORY_MANAGEMENT mode (see below), mem_SetCheck sets
*         the memory check flag.  If the memory check flag is non-zero,
*         then every memory pointer is checked for validity at the
*         beginning of every memory allocation and free call by calling
*         mem_Check.
*
*    mem_Check:
*
*         Use: mem_Check ()
*
*         If in MEMORY_MANAGEMENT mode (see below), mem_Check will
*         check every memory pointer for validity.  If an invalid
*         pointer is found, then mem_Check causes a core dump.
*
*    mem_Info:
*
*         Use: mem_Info ()
*
*         If in MEMORY_MANAGEMENT mode (see below), mem_Info will output
*         the pointer value, length, the file allocated in, the line
*         number allocated at, and the validity of the header and trailer
*         for every pointer that has been allocated and not yet freed.
*
*    The following macros define the way this module is compiled:
*
*    MEMORY_MANAGEMENT
*
*         If defined, then the debug aids that this module contains are
*         included in the memory functions.  If not defined, then the memory
*         call the system memory without storing information about the
*         memory pointers functions directly.  If memory runs out while
*         in MEMORY_MANAGEMENT mode, an abort is caused.
*
*    PROTECTED
*
*         If defined, then the information stored about memory pointers
*         is stored in static memory.  If the static is used up, then the
*         information is stored in a dynamically allocated array.
*
*    MAX_POINTERS
*
*         Defines how many pointers can be stored in the PROTECTED static
*         memory.  The default is 10000.
*
*    INITIAL_SIZE
*
*         Defines the initial size of the pointer list when the module is
*         run in an unPROTECTED mode.  INITIAL_SIZE also defines the initial
*         size of the dynamically allocated overflow list when run in a
*         PROTECTED mode and the protected memory runs out.  The default
*         is 1000.
*
*    SAFETY_BUFFER_SIZE
*
*         Defines how many extra bytes are added to the header and trailer
*         to provide a safety buffer around the allocated memory.  The
*         default is 4.
*
*    OUT_OF_MEM_ABORT
*
*         If defined, then a core dump is forced if memory runs out when
*         not in MEMORY_MANAGEMENT mode.
*
*    CONTENTS:
*
*               #include "memlib.h"
*
*               char *mem_Calloc (n, s, file, line)
*               unsigned  n;
*               unsigned  s;
*               char     *file;
*               int       line;
*
*               char *mem_Malloc (s, file, line)
*               unsigned  s;
*               char     *file;
*               int       line;
*
*               char *mem_Realloc (m, s, file, line)
*               char     *m;
*               unsigned  s;
*               char     *file;
*               int       line;
*
*               void mem_Free (m, file, line)
*               char *m;
*               char *file;
*               int   line;
*
*               char *mem_FreeNull (m, file, line)
*               char *m;
*               char *file;
*               int   line;
*
*               int mem_SetCheck (flag)
*               int flag;
*
*               void mem_Check ()
*
*               void mem_Info ()
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*                                                          Documentation
*                                                             Updated?
*    Who       When       Why                               Pcode InLine
*    --------- ---------- --------------------------------- ----- ------
*    MGS       02/01/90   Original creation.                  N     Y
*    MGS       04/05/90   Change the way the trailer is set   N     Y
*                         to eliminate word alignment
*                         problem on a SPARC.
*H-*/
#ifndef lint
static char sccsid [] = "@(#)memlib.c 1.1 91/08/02 SMI";
#endif

#include <stdio.h>
#include <string.h>
#include "memlib.h"

extern char *calloc ();
extern char *malloc ();
extern char *realloc ();
extern void  free ();

/* If memory management mode is specified... */
#ifdef MEMORY_MANAGEMENT

/* If in MEMORY_MANAGEMENT mode, abort if out of memory */
#define OUT_OF_MEM_ABORT

#ifndef SAFETY_BUFFER_SIZE
#define SAFETY_BUFFER_SIZE 4
#endif

#define INDEX_SIZE    sizeof (int)

#define HEADER_SIZE   sizeof (unsigned)
#define HEADER_VALUE  (unsigned) 1531791187

#define TRAILER_SIZE  sizeof (unsigned)
#define TRAILER_VALUE (unsigned) 1296520029

#define FRONT_SAFETY_BUFFER(ms) (char *) (ms->physical_pointer + INDEX_SIZE \
					                       + HEADER_SIZE)
#define BACK_SAFETY_BUFFER(ms) (char *) (ms->physical_pointer + \
					 ms->length + INDEX_SIZE \
					            + HEADER_SIZE \
					            + TRAILER_SIZE \
					            + SAFETY_BUFFER_SIZE)
#define INDEX(ms) *((int *) ms->physical_pointer)
#define HEADER(ms) *((unsigned *) (ms->physical_pointer + INDEX_SIZE))
#define TRAILER(ms) ((char *) (ms->physical_pointer + \
			       ms->length + INDEX_SIZE \
				          + HEADER_SIZE \
				          + SAFETY_BUFFER_SIZE))

#define FILE_NAME_LENGTH 63

/* Structure for keeping track of pointer information */
typedef struct MEMORY_STRUCT {
  char     *physical_pointer;
  char     *logical_pointer;
  unsigned  length;
  char      file [FILE_NAME_LENGTH+1];
  int       line;
} memory_struct;

/* If initial size of pointer list has not been defined... */
#ifndef INITIAL_SIZE
#define INITIAL_SIZE 1000
#endif INITIAL_SIZE

/* If protected mode is specified... */
#ifdef PROTECTED

/* If max pointers has not been defined... */
#ifndef MAX_POINTERS
#define MAX_POINTERS 10000
#endif MAX_POINTERS

static memory_struct  pointer_list [MAX_POINTERS]; /* Memory pointer list */
static int            free_list [MAX_POINTERS];    /* List of free entries */

static int            pointer_list_size = MAX_POINTERS;

static memory_struct *overflow_pointer_list;
static int           *overflow_free_list;

static int            overflow_list_size;
static int            overflow_list_length;
static int            overflow_free_list_length;

static int            overflow;

/* Else protected mode is not specified */
#else

static memory_struct *pointer_list; /* Memory pointer list */
static int           *free_list;    /* List of free entries */

static int            pointer_list_size; /* Physical size of pointer list */

#endif PROTECTED

static int            pointer_list_length; /* Length of pointer list */   
static int            free_list_length;    /* Length of free list */
static int            check_flag;          /* Memory check flag */

static void  MemoryInfo ();
static void  MemoryError ();
static void  FreePointerError ();
static int   CheckFreePointer ();
static int   CheckHeaderTrailer ();
static char *MakeMemory ();
static char *ChangeMemory ();
static void  DestroyMemory ();

void         mem_Check ();

#endif MEMORY_MANAGEMENT

#ifdef OUT_OF_MEM_ABORT
static void  OutOfMemoryError ();
#endif


char *mem_Calloc (n, s, file, line)
unsigned  n;
unsigned  s;
char     *file;
int       line;
{
  int   l = n * s;
  char *m;

  /* If the memory is allocated... */
  if (m = mem_Malloc (l, file, line))
    /* Fill the memory with 0's */
    MEMSET (m, 0, l);

  /* Return the memory pointer */
  return (m);
}


char *mem_Malloc (s, file, line)
unsigned  s;
char     *file;
int       line;
{
#ifdef MEMORY_MANAGEMENT
  /* If the check flag set... */
  if (check_flag) mem_Check ();

  /* Allocate the memory and return the pointer */
  return ((s > 0) ? MakeMemory (s, file, line) : NULL);
#else
#ifdef OUT_OF_MEM_ABORT
  char *m;

  return ((s > 0) ?
	  ((m = malloc (s)) ?
	   m : (OutOfMemoryError (file, line), (char *) NULL)) : NULL);
#else
  return ((s > 0) ? malloc (s) : NULL);
#endif
#endif
}


char *mem_Realloc (m, s, file, line)
char     *m;
unsigned  s;
char     *file;
int       line;
{
  /* If the memory pointer is NULL... */
  if (!m) return (mem_Malloc (s, file, line));

#ifdef MEMORY_MANAGEMENT
  /* If the check flag is set... */
  if (check_flag) mem_Check ();

  /* Change the length of the memory and return the pointer */
  return ((s > 0) ? ChangeMemory (m, s, file, line) : m);
#else
#ifdef OUT_OF_MEM_ABORT
  return ((s > 0) ? ((m = realloc (m, s)) ?
		     m : (OutOfMemoryError (file, line), (char *) NULL)) : m);
#else
  return ((s > 0) ? realloc (m, s) : m);
#endif
#endif
}


void mem_Free (m, file, line)
char *m;
char *file;
int   line;
{
#ifdef MEMORY_MANAGEMENT
  /* If the check flag is set... */
  if (check_flag) mem_Check ();

  /* If the memory pointer is NULL... */
  if (!m) return;

  /* If the pointer is not valid pointer to free... */
  if (CheckFreePointer (m))
    /* Output an error and die... */
    FreePointerError (m, file, line);

  /* Free up the pointer and destroy the memory structure */
  DestroyMemory (m);    
#else
  if (m) free (m);
#endif
}


char *mem_FreeNull (m, file, line)
char *m;
char *file;
int   line;
{
  /* Free up the pointer */
  mem_Free (m, file, line);

  /* Return NULL */
  return ((char *) NULL);
}


mem_SetCheck (flag)
int flag;
{
#ifdef MEMORY_MANAGEMENT
  /* Set the check flag */
  return (check_flag = flag);
#else
  return (0);
#endif
}


void mem_Check ()
{
  int check;
  int i;

#ifdef MEMORY_MANAGEMENT
  /* Loop through the pointer list... */
  for (i=0; i < pointer_list_length; i++)
    /* If the entry is used... */
    if (pointer_list [i].physical_pointer)
      /* If the header or trailer is overwritten... */
      if (check = CheckHeaderTrailer (pointer_list + i))
	/* Output an error and die... */
	MemoryError (pointer_list + i, check);

#ifdef PROTECTED
  /* Loop through the overflow pointer list... */
  for (i=0; i < overflow_list_length; i++)
    /* If the entry is used... */
    if (overflow_pointer_list [i].physical_pointer)
      /* If the header or trailer is overwritten... */
      if (check = CheckHeaderTrailer (overflow_pointer_list + i))
	/* Output an error and die... */
	MemoryError (overflow_pointer_list + i, check);
#endif
#endif
}


void mem_Info ()
{
  int i;

#ifdef MEMORY_MANAGEMENT
  /* Loop through the pointer list... */
  for (i=0; i < pointer_list_length; i++)
    /* If the entry is used... */
    if (pointer_list [i].physical_pointer)
      /* Output the contents of the memory structure */
      MemoryInfo (pointer_list + i);

#ifdef PROTECTED
  /* Loop through the overflow pointer list... */
  for (i=0; i < overflow_list_length; i++)
    /* If the entry is used... */
    if (overflow_pointer_list [i].physical_pointer)
      /* Output the contents of the memory structure */
      MemoryInfo (overflow_pointer_list + i);
#endif
#endif
}


#ifdef OUT_OF_MEM_ABORT
static void OutOfMemoryError (file, line)
char *file;
int   line;
{
  fprintf (stderr, "Out of Memory:\n\n");

  fprintf (stderr, "File: %s\n", file);
  fprintf (stderr, "Line: %d\n", line);

  /* Die... */
  abort ();
}
#endif


#ifdef MEMORY_MANAGEMENT

static char *CopyFileName (s1, s2)
char *s1;
char *s2;
{
  int l;

  /* Copy the filename */
  (void) strncpy (s1, s2, FILE_NAME_LENGTH);

  /* NULL terminate the copy */
  s1 [(((l = strlen (s2)) > FILE_NAME_LENGTH) ? FILE_NAME_LENGTH : l)] = '\0';

  /* Return the pointer to the copy */
  return (s1);
}


static char *SetHeaderTrailer (ms, index)
memory_struct *ms;
int           index;
{
  unsigned trailer = TRAILER_VALUE;
  int      i;

  /* Set the index and header of the memory */
  INDEX (ms) = index;
  HEADER (ms) = HEADER_VALUE;

  /* Set the trailer one byte at a time */
  memcpy (TRAILER (ms), (char *) &trailer, TRAILER_SIZE);

  /* Set the front and back safety buffers */
  for (i=0; i < SAFETY_BUFFER_SIZE; i++) {
    *(FRONT_SAFETY_BUFFER (ms) + i) = (char) (i & 255);
    *(BACK_SAFETY_BUFFER (ms) + i) = (char) (i & 255);
  }

  /* Return the pointer to the user's memory */
  return (ms->logical_pointer);
}


static void MemoryInfo (ms)
memory_struct *ms;
{
  int check;

  printf ("Physical Pointer     : %u\n", ms->physical_pointer);
  printf ("Logical Pointer      : %u\n", ms->logical_pointer);
  printf ("User Specified Length: %u\n", ms->length);
  printf ("File allocated in    : %s\n", ms->file);
  printf ("Line allocated at    : %d\n", ms->line);

  printf ("Header and Trailer   : %s\n\n",
	  (!(check = CheckHeaderTrailer (ms)) ?
	   "OK" : ((check < 0) ? "HEADER BAD" : "TRAILER BAD")));
}


static void MemoryError (ms, check)
memory_struct *ms;
int            check;
{
  fprintf (stderr, "%s Overwritten:\n\n",
	           ((check < 0) ? "Header" : "Trailer"));

  fprintf (stderr, "Physical Pointer     : %u\n", ms->physical_pointer);
  fprintf (stderr, "Logical Pointer      : %u\n", ms->logical_pointer);
  fprintf (stderr, "User Specified Length: %u\n", ms->length);
  fprintf (stderr, "File allocated in    : %s\n", ms->file);
  fprintf (stderr, "Line allocated at    : %d\n", ms->line);

  /* Die... */
  abort ();
}


static void FreePointerError (m, file, line)
char *m;
char *file;
int   line;
{
  fprintf (stderr, "Invalid Pointer passed to Free:\n\n");

  fprintf (stderr, "Pointer: %u\n", m);
  fprintf (stderr, "File   : %s\n", file);
  fprintf (stderr, "Line   : %d\n", line);

  /* Die... */
  abort ();
}


static int CheckFreePointer (m)
char *m;
{
  int index;

  /* If the header is bad... */
  if (*((unsigned *) ((char *) m - HEADER_SIZE
		                 - SAFETY_BUFFER_SIZE)) != HEADER_VALUE)
    return (1);

  /* Get the index for the pointer */
  index = *(int *) ((char *) m - INDEX_SIZE
		               - HEADER_SIZE
		               - SAFETY_BUFFER_SIZE);

  /* If the index is less then the pointer list length */
  if (index < pointer_list_length) {
    /* If the pointer is invalid... */
    if (m != pointer_list [index].logical_pointer)
      return (1);
  }
#ifdef PROTECTED
  /* Else if the overflow index is less then the overflow list length... */
  else if ((index - pointer_list_size > -1) &&
	   (index - pointer_list_size < overflow_list_length)) {
    /* If the pointer is invalid... */
    if (m != overflow_pointer_list [index - pointer_list_size].logical_pointer)
      return (1);
  }
#endif
  /* Else the index is bad... */
  else
    /* Output an error and die... */
    return (1);

  /* Pointer appears to be valid */
  return (0);
}


static int CheckHeaderTrailer (ms)
memory_struct *ms;
{
  unsigned trailer = TRAILER_VALUE;
  int      i;

  /* If header bad... */
  if (HEADER (ms) != HEADER_VALUE) return (-1);

  /* Check the trailer one byte at a time... */
  for (i=0; i < TRAILER_SIZE; i++)
    /* If trailer bad... */
    if (*(TRAILER (ms) + i) != *((char *) &trailer + i))
      return (1);

  /* Check the safety buffers */
  for (i=0; i < SAFETY_BUFFER_SIZE; i++) {
    /* If the front safety buffer is corrupted... */
    if (*(FRONT_SAFETY_BUFFER (ms) + i) != (char) (i & 255))
      return (-1);

    /* If the back safety buffer is corrupted... */
    if (*(BACK_SAFETY_BUFFER (ms) + i) != (char) (i & 255))
      return (1);
  }

  /* Both are OK */
  return (0);
}


static memory_struct *GetMemoryStruct (m)
char *m;
{
  /* Get the index into the pointer list */
  int index = *(int *) ((char *) m - INDEX_SIZE
			           - HEADER_SIZE
			           - SAFETY_BUFFER_SIZE);

#ifdef PROTECTED
  /* If index larger then pointer list length... */
  if (index >= pointer_list_size)
    /* Return entry from overflow list */
    return (overflow_pointer_list + index - pointer_list_size);
  else
    /* Return entry from pointer list */
    return (pointer_list + index);
#else
  /* Return entry from pointer list */
  return (pointer_list + index);
#endif
}


static char *MakeMemory (s, file, line)
unsigned  s;
char     *file;
int       line;
{
  int            index;
  memory_struct *ms;

#ifdef PROTECTED
  /* If something in the overflow free list... */
  if (overflow_free_list_length)
    /* Get an index from the overflow free list */
    index = overflow_free_list [--overflow_free_list_length];
  /* Else if something in the static free list... */
  else if (free_list_length)
    /* Get an index from the static free list */
    index = free_list [--free_list_length];
  /* Else if space left in the overflow pointer list... */
  else if (overflow_list_length < overflow_list_size)
    /* Use next entry in the overflow pointer list */
    index = overflow_list_length++ + pointer_list_size;
  /* Else if space left in the static pointer list... */
  else if (pointer_list_length < pointer_list_size)
    /* Use next entry in the static pointer list */
    index = pointer_list_length++;
  /* Else no entries left anywhere... */
  else {
    /* If no overflow lists exist... */
    if (!overflow) {
      fprintf (stderr, "Out of protected memory -\n");
      fprintf (stderr, "Allocating overflow memory...\n");

      /* Allocate the overflow lists */
      overflow_pointer_list = (memory_struct *)
	                      calloc (INITIAL_SIZE, sizeof (memory_struct));
      overflow_free_list = (int *) malloc (INITIAL_SIZE * sizeof (int));

      /* If calloc or malloc returned NULL... */
      if (!overflow_pointer_list || !overflow_free_list)
	OutOfMemoryError (file, line);

      /* Set the initial size of the overflow lists */
      overflow_list_size = INITIAL_SIZE;

      /* Set the overflow flag */
      overflow = 1;
    }
    /* Else the overflow lists exist... */
    else {
      /* Double the size of the list */
      overflow_list_size *= 2;

      /* Reallocate the overflow lists */
      overflow_pointer_list = (memory_struct *)
	                      realloc ((char *) overflow_pointer_list,
				       overflow_list_size *
				       sizeof (memory_struct));
      overflow_free_list = (int *) realloc ((char *) overflow_free_list,
					    overflow_list_size * sizeof (int));

      /* If calloc or malloc returned NULL... */
      if (!overflow_pointer_list || !overflow_free_list)
	OutOfMemoryError (file, line);
    }

    /* Use the next entry in the overflow pointer list */
    index = pointer_list_size + overflow_list_length++;
  }
#else
  /* If there is something on the free list... */
  if (free_list_length)
    /* Get an index from the free list */
    index = free_list [--free_list_length];
  /* Else if there is space left in the pointer list */
  else if (pointer_list_length < pointer_list_size)
    /* Use the next entry in the pointer list */
    index = pointer_list_length++;
  /* Else no entries left anywhere... */
  else {
    /* If the pointer list exists... */
    if (pointer_list_size) {
      /* Double the size of the lists */
      pointer_list_size *= 2;

      /* Reallocate the pointer list */
      pointer_list = (memory_struct *)
  	             realloc ((char *) pointer_list,
			      pointer_list_size * sizeof (memory_struct));

      /* Reallocate the free list */
      free_list = (int *) realloc ((char *) free_list,
				   pointer_list_size * sizeof (int));

      /* If calloc or malloc returned NULL... */
      if (!pointer_list || !free_list)
	OutOfMemoryError (file, line);
    }
    /* Else the lists do not exist... */
    else {
      /* Set the initial size of the lists */
      pointer_list_size = INITIAL_SIZE;

      /* Allocate the pointer list */
      pointer_list = (memory_struct *)
	             calloc (INITIAL_SIZE, sizeof (memory_struct));

      /* Allocate the free list */
      free_list = (int *) malloc (INITIAL_SIZE * sizeof (int));

      /* If calloc or malloc returned NULL... */
      if (!pointer_list || !free_list)
	OutOfMemoryError (file, line);
    }

    /* Use the next entry in the pointer list */
    index = pointer_list_length++;
  }
#endif    

#ifdef PROTECTED
  /* If the index is not in the static pointer list... */
  if (index >= pointer_list_length)
    /* Get the memory structure from the overflow pointer list */
    ms = overflow_pointer_list + index - pointer_list_size;
  /* Else the index is in the static pointer list... */
  else
    /* Get the memory structure from the static pointer list */
    ms = pointer_list + index;
#else
  /* Get the memory structure from the pointer list */
  ms = pointer_list + index;
#endif

  /* Initialize the memory structure */
  if (!(ms->physical_pointer = malloc (s + INDEX_SIZE
				         + HEADER_SIZE
				         + TRAILER_SIZE
				         + SAFETY_BUFFER_SIZE << 1)))
    OutOfMemoryError (file, line);

  ms->logical_pointer = ms->physical_pointer + INDEX_SIZE
                                             + HEADER_SIZE
                                             + SAFETY_BUFFER_SIZE;
  ms->length = s;
  (void) CopyFileName (ms->file, file);
  ms->line = line;

  /* Set the header and trailer and return the user's memory pointer */
  return (SetHeaderTrailer (ms, index));
}


static char *ChangeMemory (m, s, file, line)
char     *m;
unsigned  s;
char     *file;
int       line;
{
  memory_struct *ms;

  /* Get the corresponding memory structure */
  ms = GetMemoryStruct (m);

  /* Reallocate the memory and re-initialize the memory structure */
  if (!(ms->physical_pointer = realloc (ms->physical_pointer,
					s + INDEX_SIZE
					  + HEADER_SIZE
					  + TRAILER_SIZE
					  + SAFETY_BUFFER_SIZE << 1)))
    OutOfMemoryError (file, line);

  ms->logical_pointer = ms->physical_pointer + INDEX_SIZE
                                             + HEADER_SIZE
					     + SAFETY_BUFFER_SIZE;
  ms->length = s;
  (void) CopyFileName (ms->file, file);
  ms->line = line;

  /* Set the header and trailer and return the user's memory pointer */
  return (SetHeaderTrailer (ms, INDEX (ms)));
}


static void DestroyMemory (m)
char *m;
{
  memory_struct *ms;

  /* Get the corresponding memory structure */
  ms = GetMemoryStruct (m);

#ifdef PROTECTED
  /* If the is space left on the static free list... */
  if (free_list_length < pointer_list_size)
    /* Put the entry on the static free list */
    free_list [free_list_length++] = INDEX (ms);
  /* Else the static free list is used up... */
  else
    /* Put the entry on the overflow free list */
    overflow_free_list [overflow_free_list_length++] = INDEX (ms);
#else
  /* Put the entry on the free list */
  free_list [free_list_length++] = INDEX (ms);
#endif
  
  /* Free the allocated memory */
  free (ms->physical_pointer);

  /* Set the physical and logical pointers to NULL */
  ms->physical_pointer = NULL;
  ms->logical_pointer = NULL;
}

#endif


#ifdef TEST

#define SIZE      20
#define UNIT_SIZE 10

#ifndef FREE_TEST

main ()
{
  char *m [SIZE];
  char *c [SIZE];
  int   i, j;
  int   s;
#ifdef MEMORY_MANAGEMENT
#ifdef PROTECTED
  char *o [MAX_POINTERS];
#endif
#endif

  mem_SetCheck (1);

  for (i=0; i < SIZE; i++) {
    m [i] = MALLOC ((s = i * UNIT_SIZE + 1));
    for (j=0; j < s; j++)
      *(m [i] + j) = (char) (j % 256);
  }
  
  printf ("Initial Memory:\n");
  mem_Info ();

#ifdef MEMORY_MANAGEMENT
#ifdef PROTECTED

  for (i=0; i < MAX_POINTERS; i++)
    o [i] = MALLOC (i+1);

  printf ("Overflow Memory:\n");
  mem_Info ();

  for (i=0; i < MAX_POINTERS - 5; i++)
    FREE (o [i]);

  for (i=0; i < MAX_POINTERS - 5; i++)
    o [i] = MALLOC (i+1);

  printf ("Freed and Malloced Overflow Memory:\n");
  mem_Info ();

  for (i=0; i < MAX_POINTERS; i++)
    FREE (o [i]);

  printf ("Overflow Memory Freed:\n");
  mem_Info ();

#endif
#endif

  i = 14;
  m [i] = REALLOC (m [i], 2 * i * UNIT_SIZE);

  for (i=0; i < SIZE; i++)
    c [i] = CALLOC (i+1, UNIT_SIZE);

  printf ("Initial plus Calloced Memory:\n");
  mem_Info ();

  for (i=0; i < SIZE; i++)
    c [i] = FREE_NULL (c [i]);

  printf ("Calloced memory freed:\n");
  mem_Info ();

  j = 2;
  for (i=0; i < j * UNIT_SIZE + 2; i++)
    *(m [j] + i) = 0;

  printf ("Error in Memory:\n");
  mem_Info ();

  for (i=0; i < SIZE; i++)
    FREE (m [i]);
}

#else

main ()
{
  char *m [SIZE];
  int   i, j;
  int   s;

  mem_SetCheck (1);

  for (i=0; i < SIZE; i++) {
    m [i] = MALLOC ((s = i * UNIT_SIZE + 1));
    for (j=0; j < s; j++)
      *(m [i] + j) = (char) (j % 256);
  }
  
  mem_Info ();

  i = 3;
  FREE (m [i]);
  FREE (m [i]);
}

#endif
#endif
